@@ -18,10 +18,8 @@
 
 set -e
 
-cd "${GOPATH}/src/github.com/apache/trafficcontrol"
-
 download_go() {
-	go_version="$(cat GO_VERSION)"
+	go_version="$(cat "${GITHUB_WORKSPACE}/GO_VERSION")"
 	wget -O go.tar.gz "https://dl.google.com/go/go${go_version}.linux-amd64.tar.gz"
 	tar -C /usr/local -xzf go.tar.gz
 	rm go.tar.gz
@@ -36,6 +34,18 @@ if [ -z "$INPUT_DIR" ]; then
 fi
 
 # Need to fetch golang.org/x/* dependencies
-go mod vendor -v
-go test --buildvcs=false $INPUT_DIR
+if ! [ -d "${GITHUB_WORKSPACE}/vendor/golang.org" ]; then
+	go mod vendor
+fi
+
+export GOFLAGS="-buildvcs=false"
+
+# t3c-check-refs requires a binary in order to test
+# cache-config tests need to ignore the testing directory
+if [ $TEST_NAME == "cache-config" ]; then
+  go build -o ./cache-config/t3c-check-refs/ ./cache-config/t3c-check-refs/t3c-check-refs.go
+  go test $(go list $INPUT_DIR | grep -v /testing/) -coverpkg=$INPUT_DIR -coverprofile="$TEST_NAME-coverage.out"
+else
+  go test $INPUT_DIR -coverpkg=$INPUT_DIR -coverprofile="$TEST_NAME-coverage.out"
+fi
 exit $?
