@@ -26,11 +26,13 @@ import (
 	"net/url"
 	"os"
 	"os/exec"
+	"path/filepath"
+	"regexp"
 	"strings"
 	"time"
 
-	"github.com/apache/trafficcontrol/cache-config/t3cutil"
-	"github.com/apache/trafficcontrol/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/cache-config/t3cutil"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
 
 	"github.com/pborman/getopt/v2"
 )
@@ -43,7 +45,6 @@ const DefaultTSConfigDir = "/opt/trafficserver/etc/trafficserver"
 
 const (
 	StatusDir          = "/var/lib/trafficcontrol-cache-config/status"
-	GenerateCmd        = "/usr/bin/t3c-generate" // TODO don't make absolute?
 	Chkconfig          = "/sbin/chkconfig"
 	Service            = "/sbin/service"
 	SystemCtl          = "/bin/systemctl"
@@ -81,6 +82,7 @@ type Cfg struct {
 	SvcManagement       SvcManagement
 	Retries             int
 	ReverseProxyDisable bool
+	RpmDBOk             bool
 	SkipOSCheck         bool
 	UseStrategies       t3cutil.UseStrategiesFlag
 	TOInsecure          bool
@@ -108,8 +110,11 @@ type Cfg struct {
 	TsHome          string
 	TsConfigDir     string
 
-	ServiceAction     t3cutil.ApplyServiceActionFlag
+	ServiceAction          t3cutil.ApplyServiceActionFlag
+	NoConfirmServiceAction bool
+
 	ReportOnly        bool
+	GoDirect          string
 	Files             t3cutil.ApplyFilesFlag
 	InstallPackages   bool
 	IgnoreUpdateFlag  bool
@@ -117,6 +122,8 @@ type Cfg struct {
 	UpdateIPAllow     bool
 	Version           string
 	GitRevision       string
+	LocalATSVersion   string
+	CacheType         string
 }
 
 func (cfg Cfg) AppVersion() string { return t3cutil.VersionStr(AppName, cfg.Version, cfg.GitRevision) }
@@ -221,6 +228,68 @@ func GetTSPackageHome() string {
 	return tsHome
 }
 
+const (
+	rpmDBBdb             = "bdb"
+	rpmDBSquLite         = "sqlite"
+	rpmDBUnknown         = "unknown"
+	rpmDBVerifyCmd       = "/usr/lib/rpm/rpmdb_verify"
+	sqliteRpmDbVerifyCmd = "/bin/sqlite3"
+	rpmDir               = "/var/lib/rpm"
+	sqliteRpmDB          = "rpmdb.sqlite"
+)
+
+// getRpmDBBackend uses "%_db_backend" macro to get the database type
+func getRpmDBBackend() (string, error) {
+	var outBuf bytes.Buffer
+	cmd := exec.Command("/bin/rpm", "-E", "%_db_backend")
+	cmd.Stdout = &outBuf
+	err := cmd.Run()
+	if err != nil {
+		return rpmDBUnknown, err
+	}
+	return strings.TrimSpace(outBuf.String()), nil
+}
+
+// isSqliteInstalled looks to see if the sqlite3 executable
+// is installed which is needed to do the db verify
+func isSqliteInstalled() bool {
+	sqliteUtil := isCommandAvailable("/bin/sqlite3")
+	return sqliteUtil
+}
+
+// verifies the rpm database files. if there is any database corruption
+// it will return false
+func verifyRpmDB(rpmDir string) bool {
+	exclude := regexp.MustCompile(`(^\.|^__)`)
+	dbFiles, err := os.ReadDir(rpmDir)
+	if err != nil {
+		return false
+	}
+	for _, file := range dbFiles {
+		if exclude.Match([]byte(file.Name())) {
+			continue
+		}
+		cmd := exec.Command(rpmDBVerifyCmd, rpmDir+"/"+file.Name())
+		err := cmd.Run()
+		if err != nil || cmd.ProcessState.ExitCode() > 0 {
+			return false
+		}
+	}
+	return true
+}
+
+// verifySqliteRpmDB runs PRAGMA quick_check
+// requires /bin/sqlite3
+func verifySqliteRpmDB(sqliteDB string) bool {
+	args := []string{sqliteDB, `PRAGMA quick_check`}
+	cmd := exec.Command(sqliteRpmDbVerifyCmd, args...)
+	err := cmd.Run()
+	if err != nil || cmd.ProcessState.ExitCode() > 0 {
+		return false
+	}
+	return true
+}
+
 func GetCfg(appVersion string, gitRevision string) (Cfg, error) {
 	var err error
 	toInfoLog := []string{}
@@ -248,9 +317,13 @@ func GetCfg(appVersion string, gitRevision string) (Cfg, error) {
 	defaultClientTLSVersions := getopt.StringLong("default-client-tls-versions", 'V', "", "Comma-delimited list of default TLS versions for Delivery Services with no Parameter, e.g. --default-tls-versions='1.1,1.2,1.3'. If omitted, all versions are enabled.")
 	maxmindLocationPtr := getopt.StringLong("maxmind-location", 'M', "", "URL of a maxmind gzipped database file, to be installed into the trafficserver etc directory.")
 	verbosePtr := getopt.CounterLong("verbose", 'v', `Log verbosity. Logging is output to stderr. By default, errors are logged. To log warnings, pass '-v'. To log info, pass '-vv'. To omit error logging, see '-s'`)
+	cache := getopt.StringLong("cache", 'T', "ats", "Cache server type. Generate configuration files for specific cache server type, e.g. 'ats', 'varnish'.")
 	const silentFlagName = "silent"
 	silentPtr := getopt.BoolLong(silentFlagName, 's', `Silent. Errors are not logged, and the 'verbose' flag is ignored. If a fatal error occurs, the return code will be non-zero but no text will be output to stderr`)
 
+	const goDirectFlagName = "go-direct"
+	goDirectPtr := getopt.StringLong(goDirectFlagName, 'G', "old", "[true|false|old] default will set go_direct to old and it will be based on opposite of parent_is_proxy directivefalse, you can also set go_direct true, or false.")
+
 	const waitForParentsFlagName = "wait-for-parents"
 	waitForParentsPtr := getopt.BoolLong(waitForParentsFlagName, 'W', "[true | false] do not update if parent_pending = 1 in the update json. Default is false")
 
@@ -258,6 +331,9 @@ func GetCfg(appVersion string, gitRevision string) (Cfg, error) {
 	const defaultServiceAction = t3cutil.ApplyServiceActionFlagReload
 	serviceActionPtr := getopt.EnumLong(serviceActionFlagName, 'a', []string{string(t3cutil.ApplyServiceActionFlagReload), string(t3cutil.ApplyServiceActionFlagRestart), string(t3cutil.ApplyServiceActionFlagNone), ""}, "", "action to perform on Traffic Server and other system services. Only reloads if necessary, but always restarts. Default is 'reload'")
 
+	const noConfirmServiceActionFlagName = "no-confirm-service-action"
+	noConfirmServiceAction := getopt.BoolLong(noConfirmServiceActionFlagName, 0, "Whether to skip waiting and confirming the service action succeeded (reload or restart) via t3c-tail. Default is false.")
+
 	const reportOnlyFlagName = "report-only"
 	reportOnlyPtr := getopt.BoolLong(reportOnlyFlagName, 'o', "Log information about necessary files and actions, but take no action. Default is false")
 
@@ -279,6 +355,9 @@ func GetCfg(appVersion string, gitRevision string) (Cfg, error) {
 	const defaultUseStrategies = t3cutil.UseStrategiesFlagFalse
 	useStrategiesPtr := getopt.EnumLong(useStrategiesFlagName, 0, []string{string(t3cutil.UseStrategiesFlagTrue), string(t3cutil.UseStrategiesFlagCore), string(t3cutil.UseStrategiesFlagCore), ""}, "", "[true | core| false] whether to generate config using strategies.yaml instead of parent.config. If true use the parent_select plugin, if 'core' use ATS core strategies, if false use parent.config.")
 
+	const useLocalATSVersionFlagName = "local-ats-version"
+	useLocalATSVersionPtr := getopt.BoolLong(useLocalATSVersionFlagName, 0, "[true | false] whether to use the local installed ATS version for config generation. If false, attempt to use the Server Package Parameter and fall back to ATS 5. If true and the local ATS version cannot be found, an error will be logged and the version set to ATS 5. Default is false")
+
 	const runModeFlagName = "run-mode"
 	runModePtr := getopt.StringLong(runModeFlagName, 'm', "", `[badass | report | revalidate | syncds] run mode. Optional, convenience flag which sets other flags for common usage scenarios.
 syncds     keeps the defaults:
@@ -309,11 +388,13 @@ If any of the related flags are also set, they override the mode's default behav
 	// so we want to log what flags the mode set here, to aid debugging.
 	// But we can't do that until the loggers are initialized.
 	modeLogStrs := []string{}
+	fatalLogStrs := []string{}
 	if getopt.IsSet(runModeFlagName) {
 		runMode := t3cutil.StrToMode(*runModePtr)
 		if runMode == t3cutil.ModeInvalid {
-			return Cfg{}, errors.New(*runModePtr + " is an invalid mode.")
+			fatalLogStrs = append(fatalLogStrs, *runModePtr+" is an invalid mode.")
 		}
+		modeLogStrs = append(modeLogStrs, "t3c-apply is running in "+runMode.String()+" mode")
 		switch runMode {
 		case t3cutil.ModeSyncDS:
 			// syncds flags are all the defaults, no need to change anything
@@ -359,6 +440,16 @@ If any of the related flags are also set, they override the mode's default behav
 		}
 	}
 
+	switch *goDirectPtr {
+	case "false", "true", "old":
+		if !getopt.IsSet(goDirectFlagName) {
+			modeLogStrs = append(modeLogStrs, goDirectFlagName+" not set using default 'old'")
+		}
+	default:
+		modeLogStrs = append(modeLogStrs, *goDirectPtr+" is not a valid go-direct option setting default 'old'")
+		*goDirectPtr = "old"
+	}
+
 	if *serviceActionPtr == "" {
 		*serviceActionPtr = defaultServiceAction.String()
 	}
@@ -387,7 +478,7 @@ If any of the related flags are also set, they override the mode's default behav
 	}
 
 	if *verbosePtr > 2 {
-		return Cfg{}, errors.New("Too many verbose options. The maximum log verbosity level is 2 (-vv or --verbose=2) for errors (0), warnings (1), and info (2)")
+		fatalLogStrs = append(fatalLogStrs, "Too many verbose options. The maximum log verbosity level is 2 (-vv or --verbose=2) for errors (0), warnings (1), and info (2)")
 	}
 
 	var cacheHostName string
@@ -396,7 +487,7 @@ If any of the related flags are also set, they override the mode's default behav
 	} else {
 		cacheHostName, err = os.Hostname()
 		if err != nil {
-			return Cfg{}, errors.New("Could not get the hostname from the O.S., please supply a hostname: " + err.Error())
+			fatalLogStrs = append(fatalLogStrs, "Could not get the hostname from the O.S., please supply a hostname: "+err.Error())
 		}
 		// strings.Split always returns a slice with at least 1 element, so we don't need a len check
 		cacheHostName = strings.Split(cacheHostName, ".")[0]
@@ -405,7 +496,7 @@ If any of the related flags are also set, they override the mode's default behav
 	useGit := StrToUseGitFlag(*useGitStr)
 
 	if useGit == UseGitInvalid {
-		return Cfg{}, errors.New("Invalid git flag '" + *useGitStr + "'. Valid options are yes, no, auto.")
+		fatalLogStrs = append(fatalLogStrs, "Invalid git flag '"+*useGitStr+"'. Valid options are yes, no, auto.")
 	}
 
 	retries := *retriesPtr
@@ -447,6 +538,38 @@ If any of the related flags are also set, they override the mode's default behav
 		os.Setenv("TO_PASS", toPass)
 	}
 
+	rpmDBisOk := true
+	rpmDBType, err := getRpmDBBackend()
+	if err != nil {
+		toInfoLog = append(toInfoLog, fmt.Sprintf("error getting db type: %s", err.Error()))
+		rpmDBType = rpmDBUnknown
+	}
+
+	if rpmDBType == rpmDBSquLite {
+		sqliteUtil := isSqliteInstalled()
+		toInfoLog = append(toInfoLog, fmt.Sprintf("RPM database is %s", rpmDBSquLite))
+		if sqliteUtil {
+			rpmDBisOk = verifySqliteRpmDB(rpmDir + "/" + sqliteRpmDB)
+			toInfoLog = append(toInfoLog, fmt.Sprintf("RPM database is ok: %t", rpmDBisOk))
+		} else {
+			toInfoLog = append(toInfoLog, "/bin/sqlite3 not available, RPM database not checked")
+		}
+	} else if rpmDBType == rpmDBBdb {
+		toInfoLog = append(toInfoLog, fmt.Sprintf("RPM database is %s", rpmDBBdb))
+		rpmDBisOk = verifyRpmDB(rpmDir)
+		toInfoLog = append(toInfoLog, fmt.Sprintf("RPM database is ok: %t", rpmDBisOk))
+	} else {
+		toInfoLog = append(toInfoLog, fmt.Sprintf("RPM DB type is %s DB check will be skipped", rpmDBUnknown))
+	}
+
+	if *installPackagesPtr && !rpmDBisOk {
+		if t3cutil.StrToMode(*runModePtr) == t3cutil.ModeBadAss {
+			fatalLogStrs = append(fatalLogStrs, "RPM database check failed unable to install packages cannot continue in badass mode")
+		} else {
+			fatalLogStrs = append(fatalLogStrs, "RPM database check failed unable to install packages cannot continue")
+		}
+	}
+
 	// set TSHome
 	var tsHome = ""
 	if *tsHomePtr != "" {
@@ -457,13 +580,13 @@ If any of the related flags are also set, they override the mode's default behav
 		tsHome = os.Getenv("TS_HOME") // check for the environment variable.
 		if tsHome != "" {
 			toInfoLog = append(toInfoLog, fmt.Sprintf("set TSHome from TS_HOME environment variable '%s'\n", TSHome))
-		} else { // finally check using the config file listing from the rpm package.
+		} else if rpmDBisOk { // check using the config file listing from the rpm package if rpmdb is ok.
 			tsHome = GetTSPackageHome()
 			if tsHome != "" {
 				toInfoLog = append(toInfoLog, fmt.Sprintf("set TSHome from the RPM config file  list '%s'\n", TSHome))
-			} else {
-				toInfoLog = append(toInfoLog, fmt.Sprintf("no override for TSHome was found, using the configured default: '%s'\n", TSHome))
 			}
+		} else if tsHome == "" {
+			toInfoLog = append(toInfoLog, fmt.Sprintf("no override for TSHome was found, using the configured default: '%s'\n", TSHome))
 		}
 	}
 
@@ -472,21 +595,33 @@ If any of the related flags are also set, they override the mode's default behav
 	if tsHome != "" {
 		TSHome = tsHome
 		tsConfigDir = tsHome + "/etc/trafficserver"
+		if cache != nil && *cache == "varnish" {
+			tsConfigDir = tsHome + "/etc/varnish"
+		}
 		toInfoLog = append(toInfoLog, fmt.Sprintf("TSHome: %s, TSConfigDir: %s\n", TSHome, tsConfigDir))
 	}
 
+	atsVersionStr := ""
+	if *useLocalATSVersionPtr {
+		atsVersionStr, err = GetATSVersionStr(tsHome)
+		if err != nil {
+			fatalLogStrs = append(fatalLogStrs, "getting local ATS version: "+err.Error())
+		}
+	}
+	toInfoLog = append(toInfoLog, fmt.Sprintf("ATSVersionStr: '%s'\n", atsVersionStr))
+
 	usageStr := "basic usage: t3c-apply --traffic-ops-url=myurl --traffic-ops-user=myuser --traffic-ops-password=mypass --cache-host-name=my-cache"
 	if strings.TrimSpace(toURL) == "" {
-		return Cfg{}, errors.New("Missing required argument --traffic-ops-url or TO_URL environment variable. " + usageStr)
+		fatalLogStrs = append(fatalLogStrs, "Missing required argument --traffic-ops-url or TO_URL environment variable. "+usageStr)
 	}
 	if strings.TrimSpace(toUser) == "" {
-		return Cfg{}, errors.New("Missing required argument --traffic-ops-user or TO_USER environment variable. " + usageStr)
+		fatalLogStrs = append(fatalLogStrs, "Missing required argument --traffic-ops-user or TO_USER environment variable. "+usageStr)
 	}
 	if strings.TrimSpace(toPass) == "" {
-		return Cfg{}, errors.New("Missing required argument --traffic-ops-password or TO_PASS environment variable. " + usageStr)
+		fatalLogStrs = append(fatalLogStrs, "Missing required argument --traffic-ops-password or TO_PASS environment variable. "+usageStr)
 	}
 	if strings.TrimSpace(cacheHostName) == "" {
-		return Cfg{}, errors.New("Missing required argument --cache-host-name. " + usageStr)
+		fatalLogStrs = append(fatalLogStrs, "Missing required argument --cache-host-name. "+usageStr)
 	}
 
 	toURLParsed, err := url.Parse(toURL)
@@ -507,6 +642,7 @@ If any of the related flags are also set, they override the mode's default behav
 		CacheHostName:               cacheHostName,
 		SvcManagement:               svcManagement,
 		Retries:                     retries,
+		RpmDBOk:                     rpmDBisOk,
 		ReverseProxyDisable:         reverseProxyDisable,
 		SkipOSCheck:                 skipOsCheck,
 		UseStrategies:               useStrategies,
@@ -530,21 +666,31 @@ If any of the related flags are also set, they override the mode's default behav
 		MaxMindLocation:             maxmindLocation,
 		TsHome:                      TSHome,
 		TsConfigDir:                 tsConfigDir,
-
-		ServiceAction:     t3cutil.ApplyServiceActionFlag(*serviceActionPtr),
-		ReportOnly:        *reportOnlyPtr,
-		Files:             t3cutil.ApplyFilesFlag(*filesPtr),
-		InstallPackages:   *installPackagesPtr,
-		IgnoreUpdateFlag:  *ignoreUpdateFlagPtr,
-		NoUnsetUpdateFlag: *noUnsetUpdateFlagPtr,
-		Version:           appVersion,
-		GitRevision:       gitRevision,
+		GoDirect:                    *goDirectPtr,
+		ServiceAction:               t3cutil.ApplyServiceActionFlag(*serviceActionPtr),
+		NoConfirmServiceAction:      *noConfirmServiceAction,
+		ReportOnly:                  *reportOnlyPtr,
+		Files:                       t3cutil.ApplyFilesFlag(*filesPtr),
+		InstallPackages:             *installPackagesPtr,
+		IgnoreUpdateFlag:            *ignoreUpdateFlagPtr,
+		NoUnsetUpdateFlag:           *noUnsetUpdateFlagPtr,
+		Version:                     appVersion,
+		GitRevision:                 gitRevision,
+		LocalATSVersion:             atsVersionStr,
+		CacheType:                   *cache,
 	}
 
 	if err = log.InitCfg(cfg); err != nil {
 		return Cfg{}, errors.New("Initializing loggers: " + err.Error() + "\n")
 	}
 
+	if len(fatalLogStrs) > 0 {
+		for _, str := range fatalLogStrs {
+			str = strings.TrimSpace(str)
+			log.Errorln(str)
+		}
+		return Cfg{}, errors.New("fatal error has occurred")
+	}
 	for _, str := range modeLogStrs {
 		str = strings.TrimSpace(str)
 		if str == "" {
@@ -604,6 +750,22 @@ func getOSSvcManagement() SvcManagement {
 	return _svcManager
 }
 
+func GetATSVersionStr(tsHome string) (string, error) {
+	tsPath := tsHome
+	tsPath = filepath.Join(tsPath, "bin")
+	tsPath = filepath.Join(tsPath, "traffic_server")
+
+	stdOut, stdErr, code := t3cutil.Do(`sh`, `-c`, `set -o pipefail && `+tsPath+` --version | head -1 | awk '{print $3}'`)
+	if code != 0 {
+		return "", fmt.Errorf("traffic_server --version returned code %v stderr '%v' stdout '%v'", code, string(stdErr), string(stdOut))
+	}
+	atsVersion := strings.TrimSpace(string(stdOut))
+	if atsVersion == "" {
+		return "", fmt.Errorf("traffic_server --version returned nothing, code %v stderr '%v' stdout '%v'", code, string(stdErr), string(stdOut))
+	}
+	return atsVersion, nil
+}
+
 func printConfig(cfg Cfg) {
 	// TODO add new flags
 	log.Debugf("LogLocationDebug: %s\n", cfg.LogLocationDebug)
@@ -612,6 +774,7 @@ func printConfig(cfg Cfg) {
 	log.Debugf("LogLocationWarn: %s\n", cfg.LogLocationWarn)
 	log.Debugf("CacheHostName: %s\n", cfg.CacheHostName)
 	log.Debugf("SvcManagement: %s\n", cfg.SvcManagement)
+	log.Debugf("GoDirect: %s\n", cfg.GoDirect)
 	log.Debugf("Retries: %d\n", cfg.Retries)
 	log.Debugf("ReverseProxyDisable: %t\n", cfg.ReverseProxyDisable)
 	log.Debugf("SkipOSCheck: %t\n", cfg.SkipOSCheck)
@@ -621,7 +784,10 @@ func printConfig(cfg Cfg) {
 	log.Debugf("TOPass: Pass len: '%d'\n", len(cfg.TOPass))
 	log.Debugf("TOURL: %s\n", cfg.TOURL)
 	log.Debugf("TSHome: %s\n", TSHome)
+	log.Debugf("LocalATSVersion: %s\n", cfg.LocalATSVersion)
 	log.Debugf("WaitForParents: %v\n", cfg.WaitForParents)
+	log.Debugf("ServiceAction: %v\n", cfg.ServiceAction)
+	log.Debugf("NoConfirmServiceAction: %v\n", cfg.NoConfirmServiceAction)
 	log.Debugf("YumOptions: %s\n", cfg.YumOptions)
 	log.Debugf("MaxmindLocation: %s\n", cfg.MaxMindLocation)
 }
