@@ -1,7 +1,7 @@
 package crconfigdsservers
 
 import (
-	"github.com/apache/trafficcontrol/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
 )
 
 /*
