@@ -24,14 +24,14 @@ import (
 	"fmt"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/lib/go-util"
-	"github.com/apache/trafficcontrol/traffic_monitor/cache"
-	"github.com/apache/trafficcontrol/traffic_monitor/dsdata"
-	"github.com/apache/trafficcontrol/traffic_monitor/health"
-	"github.com/apache/trafficcontrol/traffic_monitor/peer"
-	"github.com/apache/trafficcontrol/traffic_monitor/todata"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/cache"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/dsdata"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/health"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/peer"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/todata"
 )
 
 // TODO remove 'ds' and 'stat' from names
