@@ -23,12 +23,13 @@ import (
 	"testing"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-rfc"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/traffic_ops/testing/api/assert"
-	"github.com/apache/trafficcontrol/traffic_ops/testing/api/utils"
-	"github.com/apache/trafficcontrol/traffic_ops/toclientlib"
-	client "github.com/apache/trafficcontrol/traffic_ops/v4-client"
+	"github.com/apache/trafficcontrol/v8/lib/go-rfc"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	totest "github.com/apache/trafficcontrol/v8/lib/go-tc/totestv4"
+	"github.com/apache/trafficcontrol/v8/lib/go-util/assert"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/testing/api/utils"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/toclientlib"
+	client "github.com/apache/trafficcontrol/v8/traffic_ops/v4-client"
 )
 
 func TestDeliveryServices(t *testing.T) {
@@ -230,7 +231,7 @@ func TestDeliveryServices(t *testing.T) {
 					ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"tlsVersions": []string{"1.1"},
-						"typeId":      GetTypeId(t, "STEERING"),
+						"typeId":      totest.GetTypeId(t, TOSession, "STEERING"),
 						"xmlId":       "test-TLS-creation-steering",
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
@@ -254,7 +255,7 @@ func TestDeliveryServices(t *testing.T) {
 			},
 			"PUT": {
 				"BAD REQUEST when using LONG DESCRIPTION 2 and 3 fields": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"longDesc1": "long desc 1",
 						"longDesc2": "long desc 2",
@@ -263,7 +264,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"OK when VALID request": {
-					EndpointId: GetDeliveryServiceId(t, "ds2"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds2"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"maxRequestHeaderBytes": 131080,
 						"longDesc":              "something different",
@@ -281,6 +282,7 @@ func TestDeliveryServices(t *testing.T) {
 						"multiSiteOrigin":       true,
 						"orgServerFqdn":         "http://origin.example.net",
 						"protocol":              2,
+						"regional":              true,
 						"routingName":           "ccr-ds2",
 						"qStringIgnore":         0,
 						"regionalGeoBlocking":   true,
@@ -296,21 +298,21 @@ func TestDeliveryServices(t *testing.T) {
 						})),
 				},
 				"BAD REQUEST when INVALID REMAP TEXT": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"remapText": "@plugin=tslua.so @pparam=/opt/trafficserver/etc/trafficserver/remapPlugin1.lua\nline2",
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when MISSING SLICE PLUGIN SIZE": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"rangeRequestHandling": 3,
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when SLICE PLUGIN SIZE SET with INVALID RANGE REQUEST SETTING": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"rangeRequestHandling": 1,
 						"rangeSliceBlockSize":  262144,
@@ -318,7 +320,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when SLICE PLUGIN SIZE TOO SMALL": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"rangeRequestHandling": 3,
 						"rangeSliceBlockSize":  0,
@@ -326,7 +328,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when SLICE PLUGIN SIZE TOO LARGE": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"rangeRequestHandling": 3,
 						"rangeSliceBlockSize":  40000000,
@@ -334,16 +336,16 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when ADDING TOPOLOGY to CLIENT STEERING DS": {
-					EndpointId: GetDeliveryServiceId(t, "ds-client-steering"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds-client-steering"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"topology": "mso-topology",
 						"xmlId":    "ds-client-steering",
-						"typeId":   GetTypeId(t, "CLIENT_STEERING"),
+						"typeId":   totest.GetTypeId(t, TOSession, "CLIENT_STEERING"),
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when TOPOLOGY DOESNT EXIST": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"topology": "",
 						"xmlId":    "ds1",
@@ -351,24 +353,25 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when ADDING TOPOLOGY to DS with DS REQUIRED CAPABILITY": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
-						"topology": "top-for-ds-req",
-						"xmlId":    "ds1",
+						"requiredCapabilities": []string{"foo"},
+						"topology":             "top-for-ds-req",
+						"xmlId":                "ds1",
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when ADDING TOPOLOGY to DS when NO CACHES in SAME CDN as DS": {
-					EndpointId: GetDeliveryServiceId(t, "top-ds-in-cdn2"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "top-ds-in-cdn2"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
-						"cdnId":    GetCDNID(t, "cdn2")(),
+						"cdnId":    totest.GetCDNID(t, TOSession, "cdn2")(),
 						"topology": "top-with-caches-in-cdn1",
 						"xmlId":    "top-ds-in-cdn2",
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"OK when REMOVING TOPOLOGY": {
-					EndpointId: GetDeliveryServiceId(t, "ds-based-top-with-no-mids"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds-based-top-with-no-mids"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"topology": nil,
 						"xmlId":    "ds-based-top-with-no-mids",
@@ -376,7 +379,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
 				},
 				"OK when DS with TOPOLOGY updates HEADER REWRITE FIELDS": {
-					EndpointId: GetDeliveryServiceId(t, "ds-top"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds-top"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"firstHeaderRewrite": "foo",
 						"innerHeaderRewrite": "bar",
@@ -387,7 +390,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
 				},
 				"BAD REQUEST when DS with NO TOPOLOGY updates HEADER REWRITE FIELDS": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"firstHeaderRewrite": "foo",
 						"innerHeaderRewrite": "bar",
@@ -396,7 +399,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when DS with TOPOLOGY updates LEGACY HEADER REWRITE FIELDS": {
-					EndpointId: GetDeliveryServiceId(t, "ds-top"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds-top"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"edgeHeaderRewrite": "foo",
 						"midHeaderRewrite":  "bar",
@@ -406,9 +409,9 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"OK when DS with NO TOPOLOGY updates LEGACY HEADER REWRITE FIELDS": {
-					EndpointId: GetDeliveryServiceId(t, "ds2"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds2"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
-						"profileId":         GetProfileId(t, "ATS_EDGE_TIER_CACHE"),
+						"profileId":         totest.GetProfileID(t, TOSession, "ATS_EDGE_TIER_CACHE")(),
 						"edgeHeaderRewrite": "foo",
 						"midHeaderRewrite":  "bar",
 						"routingName":       "ccr-ds2",
@@ -417,7 +420,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
 				},
 				"OK when UPDATING MINOR VERSION FIELDS": {
-					EndpointId: GetDeliveryServiceId(t, "ds-test-minor-versions"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds-test-minor-versions"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"consistentHashQueryParams": []string{"d", "e", "f"},
 						"consistentHashRegex":       "foo",
@@ -439,7 +442,7 @@ func TestDeliveryServices(t *testing.T) {
 						})),
 				},
 				"BAD REQUEST when INVALID COUNTRY CODE": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"geoLimit":          2,
 						"geoLimitCountries": []string{"US", "CA", "12"},
@@ -448,7 +451,7 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when CHANGING TOPOLOGY of DS with ORG SERVERS ASSIGNED": {
-					EndpointId: GetDeliveryServiceId(t, "ds-top"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds-top"), ClientSession: TOSession,
 					RequestBody: generateDeliveryService(t, map[string]interface{}{
 						"topology": "another-topology",
 						"xmlId":    "ds-top",
@@ -456,18 +459,18 @@ func TestDeliveryServices(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when UPDATING DS OUTSIDE TENANCY": {
-					EndpointId: GetDeliveryServiceId(t, "ds3"), ClientSession: tenant4UserSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds3"), ClientSession: tenant4UserSession,
 					RequestBody:  generateDeliveryService(t, map[string]interface{}{"xmlId": "ds3"}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusForbidden)),
 				},
 				"PRECONDITION FAILED when updating with IMS & IUS Headers": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestOpts:  client.RequestOptions{Header: http.Header{rfc.IfUnmodifiedSince: {currentTimeRFC}}},
 					RequestBody:  generateDeliveryService(t, map[string]interface{}{"xmlId": "ds1"}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusPreconditionFailed)),
 				},
 				"PRECONDITION FAILED when updating with IFMATCH ETAG Header": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					RequestBody:  generateDeliveryService(t, map[string]interface{}{"xmlId": "ds1"}),
 					RequestOpts:  client.RequestOptions{Header: http.Header{rfc.IfMatch: {rfc.ETag(currentTime)}}},
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusPreconditionFailed)),
@@ -475,7 +478,7 @@ func TestDeliveryServices(t *testing.T) {
 			},
 			"DELETE": {
 				"BAD REQUEST when DELETING DS OUTSIDE TENANCY": {
-					EndpointId: GetDeliveryServiceId(t, "ds3"), ClientSession: tenant4UserSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds3"), ClientSession: tenant4UserSession,
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusForbidden)),
 				},
 			},
@@ -488,7 +491,7 @@ func TestDeliveryServices(t *testing.T) {
 			},
 			"DELIVERY SERVICES CAPACITY": {
 				"OK when VALID request": {
-					EndpointId: GetDeliveryServiceId(t, "ds1"), ClientSession: TOSession,
+					EndpointID: totest.GetDeliveryServiceId(t, TOSession, "ds1"), ClientSession: TOSession,
 					Expectations: utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
 				},
 			},
@@ -506,17 +509,17 @@ func TestDeliveryServices(t *testing.T) {
 					}
 					if val, ok := testCase.RequestOpts.QueryParameters["cdn"]; ok {
 						if _, err := strconv.Atoi(val[0]); err != nil {
-							testCase.RequestOpts.QueryParameters.Set("cdn", strconv.Itoa(GetCDNID(t, val[0])()))
+							testCase.RequestOpts.QueryParameters.Set("cdn", strconv.Itoa(totest.GetCDNID(t, TOSession, val[0])()))
 						}
 					}
 					if val, ok := testCase.RequestOpts.QueryParameters["profile"]; ok {
 						if _, err := strconv.Atoi(val[0]); err != nil {
-							testCase.RequestOpts.QueryParameters.Set("profile", strconv.Itoa(GetProfileId(t, val[0])))
+							testCase.RequestOpts.QueryParameters.Set("profile", strconv.Itoa(totest.GetProfileID(t, TOSession, val[0])()))
 						}
 					}
 					if val, ok := testCase.RequestOpts.QueryParameters["type"]; ok {
 						if _, err := strconv.Atoi(val[0]); err != nil {
-							testCase.RequestOpts.QueryParameters.Set("type", strconv.Itoa(GetTypeId(t, val[0])))
+							testCase.RequestOpts.QueryParameters.Set("type", strconv.Itoa(totest.GetTypeId(t, TOSession, val[0])))
 						}
 					}
 					if val, ok := testCase.RequestOpts.QueryParameters["tenant"]; ok {
@@ -549,21 +552,21 @@ func TestDeliveryServices(t *testing.T) {
 						})
 					case "PUT":
 						t.Run(name, func(t *testing.T) {
-							resp, reqInf, err := testCase.ClientSession.UpdateDeliveryService(testCase.EndpointId(), ds, testCase.RequestOpts)
+							resp, reqInf, err := testCase.ClientSession.UpdateDeliveryService(testCase.EndpointID(), ds, testCase.RequestOpts)
 							for _, check := range testCase.Expectations {
 								check(t, reqInf, resp.Response, resp.Alerts, err)
 							}
 						})
 					case "DELETE":
 						t.Run(name, func(t *testing.T) {
-							resp, reqInf, err := testCase.ClientSession.DeleteDeliveryService(testCase.EndpointId(), testCase.RequestOpts)
+							resp, reqInf, err := testCase.ClientSession.DeleteDeliveryService(testCase.EndpointID(), testCase.RequestOpts)
 							for _, check := range testCase.Expectations {
 								check(t, reqInf, nil, resp.Alerts, err)
 							}
 						})
 					case "DELIVERY SERVICES CAPACITY":
 						t.Run(name, func(t *testing.T) {
-							resp, reqInf, err := testCase.ClientSession.GetDeliveryServiceCapacity(testCase.EndpointId(), testCase.RequestOpts)
+							resp, reqInf, err := testCase.ClientSession.GetDeliveryServiceCapacity(testCase.EndpointID(), testCase.RequestOpts)
 							for _, check := range testCase.Expectations {
 								check(t, reqInf, nil, resp.Alerts, err)
 							}
@@ -577,7 +580,7 @@ func TestDeliveryServices(t *testing.T) {
 
 func validateDSExpectedFields(expectedResp map[string]interface{}) utils.CkReqFunc {
 	return func(t *testing.T, _ toclientlib.ReqInf, resp interface{}, _ tc.Alerts, _ error) {
-		dsResp := resp.([]tc.DeliveryServiceV40)
+		dsResp := resp.([]tc.DeliveryServiceV4)
 		for field, expected := range expectedResp {
 			for _, ds := range dsResp {
 				switch field {
@@ -657,7 +660,7 @@ func validateDSExpectedFields(expectedResp map[string]interface{}) utils.CkReqFu
 
 func validatePagination(paginationParam string) utils.CkReqFunc {
 	return func(t *testing.T, _ toclientlib.ReqInf, resp interface{}, _ tc.Alerts, _ error) {
-		paginationResp := resp.([]tc.DeliveryServiceV40)
+		paginationResp := resp.([]tc.DeliveryServiceV4)
 
 		opts := client.NewRequestOptions()
 		opts.QueryParameters.Set("orderby", "id")
@@ -679,7 +682,7 @@ func validatePagination(paginationParam string) utils.CkReqFunc {
 
 func validateDescSort() utils.CkReqFunc {
 	return func(t *testing.T, _ toclientlib.ReqInf, resp interface{}, alerts tc.Alerts, _ error) {
-		dsDescResp := resp.([]tc.DeliveryServiceV40)
+		dsDescResp := resp.([]tc.DeliveryServiceV4)
 		var descSortedList []string
 		var ascSortedList []string
 		assert.GreaterOrEqual(t, len(dsDescResp), 2, "Need at least 2 XMLIDs in Traffic Ops to test desc sort, found: %d", len(dsDescResp))
@@ -701,38 +704,12 @@ func validateDescSort() utils.CkReqFunc {
 	}
 }
 
-func GetDeliveryServiceId(t *testing.T, xmlId string) func() int {
-	return func() int {
-		opts := client.NewRequestOptions()
-		opts.QueryParameters.Set("xmlId", xmlId)
-
-		resp, _, err := TOSession.GetDeliveryServices(opts)
-		assert.RequireNoError(t, err, "Get Delivery Service Request failed with error: %v", err)
-		assert.RequireEqual(t, 1, len(resp.Response), "Expected delivery service response object length 1, but got %d", len(resp.Response))
-		assert.RequireNotNil(t, resp.Response[0].ID, "Expected id to not be nil")
-
-		return *resp.Response[0].ID
-	}
-}
-
-func GetProfileId(t *testing.T, profileName string) int {
-	opts := client.NewRequestOptions()
-	opts.QueryParameters.Set("name", profileName)
-	resp, _, err := TOSession.GetProfiles(opts)
-
-	assert.RequireNoError(t, err, "Get Profiles Request failed with error: %v", err)
-	assert.RequireEqual(t, 1, len(resp.Response), "Expected response object length 1, but got %d", len(resp.Response))
-	assert.RequireNotNil(t, &resp.Response[0].ID, "Expected id to not be nil")
-
-	return resp.Response[0].ID
-}
-
 func generateDeliveryService(t *testing.T, requestDS map[string]interface{}) map[string]interface{} {
 	// map for the most basic HTTP Delivery Service a user can create
 	genericHTTPDS := map[string]interface{}{
 		"active":               true,
 		"cdnName":              "cdn1",
-		"cdnId":                GetCDNID(t, "cdn1")(),
+		"cdnId":                totest.GetCDNID(t, TOSession, "cdn1")(),
 		"displayName":          "test ds",
 		"dscp":                 0,
 		"geoLimit":             0,
@@ -748,11 +725,12 @@ func generateDeliveryService(t *testing.T, requestDS map[string]interface{}) map
 		"profileName":          "ATS_EDGE_TIER_CACHE",
 		"qstringIgnore":        0,
 		"rangeRequestHandling": 0,
+		"regional":             false,
 		"regionalGeoBlocking":  false,
 		"routingName":          "ccr-ds1",
 		"tenant":               "tenant1",
 		"type":                 tc.DSTypeHTTP,
-		"typeId":               GetTypeId(t, "HTTP"),
+		"typeId":               totest.GetTypeId(t, TOSession, "HTTP"),
 		"xmlId":                "ds1",
 	}
 	for k, v := range requestDS {
@@ -760,31 +738,3 @@ func generateDeliveryService(t *testing.T, requestDS map[string]interface{}) map
 	}
 	return genericHTTPDS
 }
-
-func CreateTestDeliveryServices(t *testing.T) {
-	for _, ds := range testData.DeliveryServices {
-		ds = ds.RemoveLD1AndLD2()
-		if ds.XMLID == nil {
-			t.Error("Found a Delivery Service in testing data with null or undefined XMLID")
-			continue
-		}
-		resp, _, err := TOSession.CreateDeliveryService(ds, client.RequestOptions{})
-		assert.NoError(t, err, "Could not create Delivery Service '%s': %v - alerts: %+v", *ds.XMLID, err, resp.Alerts)
-	}
-}
-
-func DeleteTestDeliveryServices(t *testing.T) {
-	dses, _, err := TOSession.GetDeliveryServices(client.RequestOptions{})
-	assert.NoError(t, err, "Cannot get Delivery Services: %v - alerts: %+v", err, dses.Alerts)
-
-	for _, ds := range dses.Response {
-		delResp, _, err := TOSession.DeleteDeliveryService(*ds.ID, client.RequestOptions{})
-		assert.NoError(t, err, "Could not delete Delivery Service: %v - alerts: %+v", err, delResp.Alerts)
-		// Retrieve Delivery Service to see if it got deleted
-		opts := client.NewRequestOptions()
-		opts.QueryParameters.Set("id", strconv.Itoa(*ds.ID))
-		getDS, _, err := TOSession.GetDeliveryServices(opts)
-		assert.NoError(t, err, "Error deleting Delivery Service for '%s' : %v - alerts: %+v", *ds.XMLID, err, getDS.Alerts)
-		assert.Equal(t, 0, len(getDS.Response), "Expected Delivery Service '%s' to be deleted", *ds.XMLID)
-	}
-}
