@@ -32,11 +32,13 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-rfc"
-	"github.com/apache/trafficcontrol/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-rfc"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
 )
 
+const nilVersionErrorMsg = "a wrapped handler was called without an API version"
+
 type KeyFieldInfo struct {
 	Field string
 	Func  func(string) (interface{}, error)
@@ -142,12 +144,14 @@ func SetLastModifiedHeader(r *http.Request, useIMS bool) bool {
 type errWriterFunc func(w http.ResponseWriter, r *http.Request, tx *sql.Tx, statusCode int, userErr error, sysErr error)
 type readSuccessWriterFunc func(w http.ResponseWriter, r *http.Request, statusCode int, results interface{})
 type deleteSuccessWriterFunc func(w http.ResponseWriter, r *http.Request, message string)
+type createSuccessWriterFunc func(w http.ResponseWriter, r *http.Request, statusCode int, alerts tc.Alerts, results interface{})
 
 // ReadHandler creates a handler function from the pointer to a struct implementing the Reader interface
-//      this handler retrieves the user from the context
-//      combines the path and query parameters
-//      produces the proper status code based on the error code returned
-//      marshals the structs returned into the proper response json
+//
+//	this handler retrieves the user from the context
+//	combines the path and query parameters
+//	produces the proper status code based on the error code returned
+//	marshals the structs returned into the proper response json
 func ReadHandler(reader Reader) http.HandlerFunc {
 	return readHandlerHelper(
 		reader,
@@ -218,12 +222,13 @@ func readHandlerHelper(reader Reader, errHandler errWriterFunc, successHandler r
 }
 
 // UpdateHandler creates a handler function from the pointer to a struct implementing the Updater interface
-//   this generic handler encapsulates the logic for handling:
-//   *fetching the id from the path parameter
-//   *current user
-//   *decoding and validating the struct
-//   *change log entry
-//   *forming and writing the body over the wire
+//
+//	this generic handler encapsulates the logic for handling:
+//	*fetching the id from the path parameter
+//	*current user
+//	*decoding and validating the struct
+//	*change log entry
+//	*forming and writing the body over the wire
 func UpdateHandler(updater Updater) http.HandlerFunc {
 	return func(w http.ResponseWriter, r *http.Request) {
 		inf, userErr, sysErr, errCode := NewInfo(r, nil, nil)
@@ -314,11 +319,12 @@ func UpdateHandler(updater Updater) http.HandlerFunc {
 }
 
 // DeleteHandler creates a handler function from the pointer to a struct implementing the Deleter interface
-//   this generic handler encapsulates the logic for handling:
-//   *fetching the id from the path parameter
-//   *current user
-//   *change log entry
-//   *forming and writing the body over the wire
+//
+//	this generic handler encapsulates the logic for handling:
+//	*fetching the id from the path parameter
+//	*current user
+//	*change log entry
+//	*forming and writing the body over the wire
 func DeleteHandler(deleter Deleter) http.HandlerFunc {
 	return deleteHandlerHelper(
 		deleter,
@@ -336,11 +342,12 @@ func DeleteHandler(deleter Deleter) http.HandlerFunc {
 }
 
 // DeprecatedDeleteHandler creates a handler function from the pointer to a struct implementing the Deleter interface with a optional deprecation notice
-//   this generic handler encapsulates the logic for handling:
-//   *fetching the id from the path parameter
-//   *current user
-//   *change log entry
-//   *forming and writing the body over the wire
+//
+//	this generic handler encapsulates the logic for handling:
+//	*fetching the id from the path parameter
+//	*current user
+//	*change log entry
+//	*forming and writing the body over the wire
 func DeprecatedDeleteHandler(deleter Deleter, alternative *string) http.HandlerFunc {
 	return deleteHandlerHelper(
 		deleter,
@@ -460,23 +467,40 @@ func deleteHandlerHelper(deleter Deleter, errHandler errWriterFunc, successHandl
 }
 
 // CreateHandler creates a handler function from the pointer to a struct implementing the Creator interface
-//   this generic handler encapsulates the logic for handling:
-//   *current user
-//   *decoding and validating the struct
-//   *change log entry
-//   *forming and writing the body over the wire
+//
+//	this generic handler encapsulates the logic for handling:
+//	*current user
+//	*decoding and validating the struct
+//	*change log entry
+//	*forming and writing the body over the wire
 func CreateHandler(creator Creator) http.HandlerFunc {
+	return createHandlerHelper(
+		creator,
+		HandleErr,
+		func(w http.ResponseWriter, r *http.Request, statusCode int, alerts tc.Alerts, results interface{}) {
+			if len(alerts.Alerts) > 0 {
+				WriteAlertsObj(w, r, statusCode, alerts, results)
+			} else {
+				w.WriteHeader(statusCode)
+				WriteResp(w, r, results)
+			}
+
+		},
+	)
+}
+
+func createHandlerHelper(creator Creator, errHandler errWriterFunc, successHandler createSuccessWriterFunc) http.HandlerFunc {
 	return func(w http.ResponseWriter, r *http.Request) {
 		inf, userErr, sysErr, errCode := NewInfo(r, nil, nil)
 		if userErr != nil || sysErr != nil {
-			HandleErr(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
+			errHandler(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
 			return
 		}
 		defer inf.Close()
 
 		interfacePtr := reflect.ValueOf(creator)
 		if interfacePtr.Kind() != reflect.Ptr {
-			HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("reflect: can only indirect from a pointer"))
+			errHandler(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("reflect: can only indirect from a pointer"))
 			return
 		}
 		objectType := reflect.Indirect(interfacePtr).Type()
@@ -486,13 +510,18 @@ func CreateHandler(creator Creator) http.HandlerFunc {
 		if c, ok := obj.(MultipleCreator); ok && c.AllowMultipleCreates() {
 			data, err := ioutil.ReadAll(r.Body)
 			if err != nil {
-				HandleErr(w, r, inf.Tx.Tx, http.StatusBadRequest, err, nil)
+				errHandler(w, r, inf.Tx.Tx, http.StatusBadRequest, err, nil)
+				return
+			}
+
+			if len(data) == 0 {
+				HandleErr(w, r, inf.Tx.Tx, http.StatusBadRequest, errors.New("no request body supplied"), nil)
 				return
 			}
 
 			objSlice, err := parseMultipleCreates(data, objectType, inf)
 			if err != nil {
-				HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, err)
+				errHandler(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, err)
 				return
 			}
 
@@ -505,30 +534,30 @@ func CreateHandler(creator Creator) http.HandlerFunc {
 					if sysErr != nil {
 						code = http.StatusInternalServerError
 					}
-					HandleErr(w, r, inf.Tx.Tx, code, userErr, sysErr)
+					errHandler(w, r, inf.Tx.Tx, code, userErr, sysErr)
 					return
 				}
 
 				if t, ok := objElem.(Tenantable); ok {
 					authorized, err := t.IsTenantAuthorized(inf.User)
 					if err != nil {
-						HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("checking tenant authorized: "+err.Error()))
+						errHandler(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("checking tenant authorized: "+err.Error()))
 						return
 					}
 					if !authorized {
-						HandleErr(w, r, inf.Tx.Tx, http.StatusForbidden, errors.New("not authorized on this tenant"), nil)
+						errHandler(w, r, inf.Tx.Tx, http.StatusForbidden, errors.New("not authorized on this tenant"), nil)
 						return
 					}
 				}
 
 				userErr, sysErr, errCode = objElem.Create()
 				if userErr != nil || sysErr != nil {
-					HandleErr(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
+					errHandler(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
 					return
 				}
 
 				if err = CreateChangeLog(ApiChange, Created, objElem, inf.User, inf.Tx.Tx); err != nil {
-					HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, fmt.Errorf("inserting changelog: %w", err))
+					errHandler(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, fmt.Errorf("inserting changelog: %w", err))
 					return
 				}
 			}
@@ -552,7 +581,7 @@ func CreateHandler(creator Creator) http.HandlerFunc {
 					alerts.AddAlerts(objElem.(AlertsResponse).GetAlerts())
 				}
 			}
-			WriteAlertsObj(w, r, http.StatusOK, alerts, responseObj)
+			successHandler(w, r, http.StatusOK, alerts, responseObj)
 
 		} else {
 			userErr, sysErr := decodeAndValidateRequestBody(r, obj)
@@ -561,41 +590,58 @@ func CreateHandler(creator Creator) http.HandlerFunc {
 				if sysErr != nil {
 					code = http.StatusInternalServerError
 				}
-				HandleErr(w, r, inf.Tx.Tx, code, userErr, sysErr)
+				errHandler(w, r, inf.Tx.Tx, code, userErr, sysErr)
 				return
 			}
 
 			if t, ok := obj.(Tenantable); ok {
 				authorized, err := t.IsTenantAuthorized(inf.User)
 				if err != nil {
-					HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("checking tenant authorized: "+err.Error()))
+					errHandler(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("checking tenant authorized: "+err.Error()))
 					return
 				}
 				if !authorized {
-					HandleErr(w, r, inf.Tx.Tx, http.StatusForbidden, errors.New("not authorized on this tenant"), nil)
+					errHandler(w, r, inf.Tx.Tx, http.StatusForbidden, errors.New("not authorized on this tenant"), nil)
 					return
 				}
 			}
 
 			userErr, sysErr, errCode = obj.Create()
 			if userErr != nil || sysErr != nil {
-				HandleErr(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
+				errHandler(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
 				return
 			}
 
 			if err := CreateChangeLog(ApiChange, Created, obj, inf.User, inf.Tx.Tx); err != nil {
-				HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, fmt.Errorf("inserting changelog: %w", err))
+				errHandler(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, fmt.Errorf("inserting changelog: %w", err))
 				return
 			}
 			alerts := tc.CreateAlerts(tc.SuccessLevel, obj.GetType()+" was created.")
 			if alertsObj, hasAlerts := obj.(AlertsResponse); hasAlerts {
 				alerts.AddAlerts(alertsObj.GetAlerts())
 			}
-			WriteAlertsObj(w, r, http.StatusOK, alerts, obj)
+			successHandler(w, r, http.StatusOK, alerts, obj)
 		}
 	}
 }
 
+// DeprecatedCreateHandler creates a net/http.HandlerFunc for the passed Creator object, and adds a deprecation
+// notice, optionally with a passed alternative route suggestion.
+func DeprecatedCreateHandler(creator Creator, alternative *string) http.HandlerFunc {
+	return createHandlerHelper(
+		creator,
+		func(w http.ResponseWriter, r *http.Request, tx *sql.Tx, statusCode int, userErr error, sysErr error) {
+			HandleDeprecatedErr(w, r, tx, statusCode, userErr, sysErr, alternative)
+		},
+		func(w http.ResponseWriter, r *http.Request, statusCode int, alerts tc.Alerts, results interface{}) {
+			depAlerts := CreateDeprecationAlerts(alternative)
+			al := tc.Alerts{Alerts: depAlerts.Alerts}
+			al.AddAlerts(alerts)
+			WriteAlertsObj(w, r, statusCode, al, results)
+		},
+	)
+}
+
 func parseMultipleCreates(data []byte, desiredType reflect.Type, inf *APIInfo) ([]Creator, error) {
 	buf := ioutil.NopCloser(bytes.NewReader(data))
 
@@ -631,3 +677,41 @@ func parseMultipleCreates(data []byte, desiredType reflect.Type, inf *APIInfo) (
 
 	return creatorSlice, nil
 }
+
+// A Handler is an API endpoint handlers. The take in APIInfo helper objects and
+// return - in order - an HTTP response status code, a user-facing error (if one
+// occurred), and a system-only error not safe for exposure to clients (if one
+// occurred).
+type Handler = func(*APIInfo) (int, error, error)
+
+// Wrap wraps an API endpoint handler in the more generic HTTP request handler
+// type from the http package. This constructs and provides the APIInfo for the
+// underlying Handler. If the handler requires any request path and/or query
+// string parameters, those should be declared in requiredParams. Likewise, if
+// any of those parameters are required to be integral, they should be named in
+// intParams.
+// Note that this will still require the normal routing middleware for
+// authentication and context setup.
+// Also note that handlers utilizing this need not defer closing of the provided
+// APIInfo, as this will handle that for them.
+// Finally, make sure this is ONLY used on versioned endpoints; this will return
+// an internal error if there is no associated API version.
+func Wrap(h Handler, requiredParams, intParams []string) http.HandlerFunc {
+	return func(w http.ResponseWriter, r *http.Request) {
+		inf, userErr, sysErr, errCode := NewInfo(r, requiredParams, intParams)
+		if userErr != nil || sysErr != nil {
+			HandleErr(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
+			return
+		}
+		defer inf.Close()
+		if inf.Version == nil {
+			HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New(nilVersionErrorMsg))
+		}
+		inf.w = w
+
+		errCode, userErr, sysErr = h(inf)
+		if userErr != nil || sysErr != nil {
+			HandleErr(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
+		}
+	}
+}
