@@ -31,9 +31,9 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-rfc"
-	"github.com/apache/trafficcontrol/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-rfc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
 )
 
 // Options is a structure used to hold the route configuration options that can be supplied for the backend routes.
@@ -70,7 +70,6 @@ type Config struct {
 	URL                                       *url.URL `json:"-"`
 	CertPath                                  string   `json:"-"`
 	KeyPath                                   string   `json:"-"`
-	ConfigHypnotoad                           `json:"hypnotoad"`
 	ConfigTrafficOpsGolang                    `json:"traffic_ops_golang"`
 	ConfigTO                                  *ConfigTO   `json:"to"`
 	SMTP                                      *ConfigSMTP `json:"smtp"`
@@ -95,12 +94,7 @@ type Config struct {
 	RoleBasedPermissions                      bool                    `json:"role_based_permissions"`
 	DefaultCertificateInfo                    *DefaultCertificateInfo `json:"default_certificate_info"`
 	Cdni                                      *CdniConf               `json:"cdni"`
-}
-
-// ConfigHypnotoad carries http setting for hypnotoad (mojolicious) server
-type ConfigHypnotoad struct {
-	Listen []string `json:"listen"`
-	// NOTE: don't care about any other fields for now
+	ClientCertAuth                            *ClientCertAuth         `json:"client_certificate_authentication"`
 }
 
 // ConfigTrafficOpsGolang carries settings specific to traffic_ops_golang server
@@ -108,6 +102,9 @@ type ConfigTrafficOpsGolang struct {
 	// Deprecated in 5.0
 	Insecure bool `json:"insecure"`
 	// end deprecated
+	//Moved from Hypnotoad-listen section
+	Cert                     string                     `json:"cert"`
+	Key                      string                     `json:"key"`
 	Port                     string                     `json:"port"`
 	ProxyTimeout             int                        `json:"proxy_timeout"`
 	ProxyKeepAlive           int                        `json:"proxy_keep_alive"`
@@ -136,6 +133,7 @@ type ConfigTrafficOpsGolang struct {
 	RiakPort             *uint    `json:"riak_port"`
 	WhitelistedOAuthUrls []string `json:"whitelisted_oauth_urls"`
 	OAuthClientSecret    string   `json:"oauth_client_secret"`
+	OAuthUserAttribute   string   `json:"oauth_user_attribute"`
 	RoutingBlacklist     `json:"routing_blacklist"`
 	SupportedDSMetrics   []string        `json:"supported_ds_metrics"`
 	TLSConfig            *tls.Config     `json:"tls_config"`
@@ -274,6 +272,10 @@ type CdniConf struct {
 	DCdnId string `json:"dcdn_id"`
 }
 
+type ClientCertAuth struct {
+	RootCertsDir string `json:"root_certificates_directory"`
+}
+
 // NewFakeConfig returns a fake Config struct with just enough data to view Routes.
 func NewFakeConfig() Config {
 	c := Config{}
@@ -432,20 +434,20 @@ func LoadConfig(cdnConfPath string, dbConfPath string, appVersion string) (Confi
 	return cfg, []error{}, AllowStartup
 }
 
-// GetCertPath - extracts path to cert .cert file
+// GetCertPath - get the path to cert .cert file
 func (c Config) GetCertPath() string {
-	v, ok := c.URL.Query()["cert"]
-	if ok {
-		return v[0]
+	cert := c.Cert
+	if cert != "" {
+		return cert
 	}
 	return ""
 }
 
-// GetKeyPath - extracts path to cert .key file
+// GetKeyPath - get the path to cert .key file
 func (c Config) GetKeyPath() string {
-	v, ok := c.URL.Query()["key"]
-	if ok {
-		return v[0]
+	key := c.Key
+	if key != "" {
+		return key
 	}
 	return ""
 }
@@ -458,6 +460,12 @@ const (
 // ParseConfig validates required fields, and parses non-JSON types
 func ParseConfig(cfg Config) (Config, error) {
 	missings := ""
+	if cfg.Cert == "" {
+		missings += `"cert", `
+	}
+	if cfg.Key == "" {
+		missings += `"key", `
+	}
 	if cfg.Port == "" {
 		missings += "port, "
 	}
@@ -497,19 +505,15 @@ func ParseConfig(cfg Config) (Config, error) {
 
 	invalidTOURLStr := ""
 	var err error
-	if len(cfg.Listen) < 1 {
-		missings += `"listen", `
-	} else {
-		listen := cfg.Listen[0]
-		if cfg.URL, err = url.Parse(listen); err != nil {
-			invalidTOURLStr = fmt.Sprintf("invalid Traffic Ops URL '%s': %v", listen, err)
-		}
-		cfg.KeyPath = cfg.GetKeyPath()
-		cfg.CertPath = cfg.GetCertPath()
-
-		newURL := url.URL{Scheme: cfg.URL.Scheme, Host: cfg.URL.Host, Path: cfg.URL.Path}
-		cfg.URL = &newURL
+	rawURL := fmt.Sprintf("cert=%s&key=%s", cfg.Cert, cfg.Key)
+	if cfg.URL, err = url.Parse(rawURL); err != nil {
+		invalidTOURLStr = fmt.Sprintf("invalid Traffic Ops URL '%s': %v", rawURL, err)
 	}
+	cfg.KeyPath = cfg.GetKeyPath()
+	cfg.CertPath = cfg.GetCertPath()
+
+	newURL := url.URL{Scheme: "https", Host: cfg.URL.Host}
+	cfg.URL = &newURL
 
 	if cfg.ConfigTO == nil {
 		missings += "to, "
@@ -534,10 +538,6 @@ func ParseConfig(cfg Config) (Config, error) {
 		errStr += "; "
 	}
 	errStr += invalidTOURLStr
-	if errStr != "" {
-		return Config{}, fmt.Errorf(errStr)
-	}
-
 	if err := ValidateRoutingBlacklist(cfg.RoutingBlacklist); err != nil {
 		return Config{}, err
 	}
