@@ -1,3 +1,5 @@
+// Package request contains logic and handlers for API routes dealing with
+// Delivery Service Requests (DSRs).
 package request
 
 /*
@@ -28,17 +30,17 @@ import (
 	"strings"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-rfc"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-rfc"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
 
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/api"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/dbhelpers"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/deliveryservice"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/routing/middleware"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/tenant"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/util/ims"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/api"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/dbhelpers"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/deliveryservice"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/routing/middleware"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/tenant"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/util/ims"
 
 	"github.com/jmoiron/sqlx"
 	"github.com/lib/pq"
@@ -110,7 +112,7 @@ FROM deliveryservice_request
 WHERE id=$1
 `
 
-//TODO: figure out how to modify 'AddTenancyCheck' so this isn't necessary
+// TODO: figure out how to modify 'AddTenancyCheck' so this isn't necessary.
 const customTenancyCheck = `(
 	CASE r.change_type
 	WHEN 'delete' THEN CAST(r.original->>'tenantId' AS BIGINT) = ANY(CAST(:accessibleTenants AS BIGINT[]))
@@ -136,23 +138,20 @@ func selectMaxLastUpdatedQuery(where string) string {
 // them as originals on the Delivery Services to which each ID maps in
 // needOriginals. It returns a response code to use if an error occurred, in
 // which case it also returns a user error and a system error.
-func getOriginals(ids []int, tx *sqlx.Tx, needOriginals map[int][]*tc.DeliveryServiceRequestV4, omitExtraLongDescFields bool) (int, error, error) {
+func getOriginals(ids []int, tx *sqlx.Tx, needOriginals map[int][]*tc.DeliveryServiceRequestV5) (int, error, error) {
 	if len(ids) > 0 {
 		originals, userErr, sysErr, errCode := deliveryservice.GetDeliveryServices(originalsQuery, map[string]interface{}{"ids": pq.Array(ids)}, tx)
 		if userErr != nil || sysErr != nil {
 			return errCode, userErr, sysErr
 		}
 
-		for _, original := range originals {
-			if original.ID == nil {
+		for _, ds := range originals {
+			if original := ds.DS; original.ID == nil {
 				log.Warnf("Trying to fill in originals: found Delivery Service with no ID")
 			} else if need, ok := needOriginals[*original.ID]; ok {
 				for _, n := range need {
-					n.Original = new(tc.DeliveryServiceV4)
+					n.Original = new(tc.DeliveryServiceV5)
 					*n.Original = original
-					if omitExtraLongDescFields {
-						*n.Original = n.Original.RemoveLD1AndLD2()
-					}
 				}
 			} else {
 				log.Warnf("Trying to fill in originals: found Delivery Service that wasn't identified by a DSR (#%d)", *original.ID)
@@ -201,13 +200,8 @@ func Get(w http.ResponseWriter, r *http.Request) {
 
 	// TODO: add this functionality to the query builder in dbhelpers
 	if xmlID, ok := inf.Params["xmlId"]; ok {
+		where = dbhelpers.AppendWhere(where, "((r.deliveryservice->>'xmlId' = :xmlId) OR (r.original->>'xmlId' = :xmlId))")
 		queryValues["xmlId"] = xmlID
-		if where != "" {
-			where += " AND "
-		} else {
-			where = "WHERE "
-		}
-		where += "(r.deliveryservice->>'xmlId' = :xmlId) OR (r.original->>'xmlId' = :xmlId)"
 	}
 
 	var maxTime *time.Time
@@ -227,17 +221,12 @@ func Get(w http.ResponseWriter, r *http.Request) {
 
 	tenantIDs, err := tenant.GetUserTenantIDListTx(tx, inf.User.TenantID)
 	if err != nil {
-		sysErr = fmt.Errorf("dsr getting tenant list: %v", err)
+		sysErr = fmt.Errorf("dsr getting tenant list: %w", err)
 		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
 		return
 	}
 
-	if where == "" {
-		where = "WHERE "
-	} else {
-		where += " AND "
-	}
-	where += customTenancyCheck
+	where = dbhelpers.AppendWhere(where, customTenancyCheck)
 	queryValues["accessibleTenants"] = pq.Array(tenantIDs)
 
 	query := selectQuery + where + orderBy + pagination
@@ -245,19 +234,19 @@ func Get(w http.ResponseWriter, r *http.Request) {
 
 	rows, err := inf.Tx.NamedQuery(query, queryValues)
 	if err != nil {
-		sysErr = fmt.Errorf("dsr querying: %v", err)
+		sysErr = fmt.Errorf("dsr querying: %w", err)
 		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
 		return
 	}
-	defer rows.Close()
+	defer log.Close(rows, "getting DSRs")
 
-	dsrs := []tc.DeliveryServiceRequestV40{}
-	needOriginals := map[int][]*tc.DeliveryServiceRequestV40{}
+	dsrs := []tc.DeliveryServiceRequestV5{}
+	needOriginals := map[int][]*tc.DeliveryServiceRequestV5{}
 	var originalIDs []int
 	for rows.Next() {
-		var dsr tc.DeliveryServiceRequestV40
+		var dsr tc.DeliveryServiceRequestV5
 		if err = rows.StructScan(&dsr); err != nil {
-			api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, fmt.Errorf("dsr scanning: %v", err))
+			api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, fmt.Errorf("dsr scanning: %w", err))
 			return
 		}
 		dsrs = append(dsrs, dsr)
@@ -266,7 +255,7 @@ func Get(w http.ResponseWriter, r *http.Request) {
 			if dsr.ChangeType == tc.DSRChangeTypeUpdate && dsr.Requested != nil && dsr.Requested.ID != nil {
 				id := *dsr.Requested.ID
 				if _, ok := needOriginals[id]; !ok {
-					needOriginals[id] = []*tc.DeliveryServiceRequestV40{&dsrs[len(dsrs)-1]}
+					needOriginals[id] = []*tc.DeliveryServiceRequestV5{&dsrs[len(dsrs)-1]}
 				} else {
 					needOriginals[id] = append(needOriginals[id], &dsrs[len(dsrs)-1])
 				}
@@ -274,7 +263,7 @@ func Get(w http.ResponseWriter, r *http.Request) {
 			} else if dsr.ChangeType == tc.DSRChangeTypeDelete && dsr.Original != nil && dsr.Original.ID != nil {
 				id := *dsr.Original.ID
 				if _, ok := needOriginals[id]; !ok {
-					needOriginals[id] = []*tc.DeliveryServiceRequestV40{&dsrs[len(dsrs)-1]}
+					needOriginals[id] = []*tc.DeliveryServiceRequestV5{&dsrs[len(dsrs)-1]}
 				} else {
 					needOriginals[id] = append(needOriginals[id], &dsrs[len(dsrs)-1])
 				}
@@ -288,18 +277,34 @@ func Get(w http.ResponseWriter, r *http.Request) {
 	}
 
 	if version.Major >= 4 {
-		errCode, userErr, sysErr = getOriginals(originalIDs, inf.Tx, needOriginals, true)
+		errCode, userErr, sysErr = getOriginals(originalIDs, inf.Tx, needOriginals)
 		if userErr != nil || sysErr != nil {
 			api.HandleErr(w, r, tx, errCode, userErr, sysErr)
 			return
 		}
-		api.WriteResp(w, r, dsrs)
+		if version.Major >= 5 {
+			api.WriteResp(w, r, dsrs)
+			return
+		}
+		if version.Minor >= 1 {
+			downgraded := make([]tc.DeliveryServiceRequestV4, 0, len(dsrs))
+			for _, dsr := range dsrs {
+				downgraded = append(downgraded, dsr.Downgrade())
+			}
+			api.WriteResp(w, r, downgraded)
+			return
+		}
+		downgraded := make([]tc.DeliveryServiceRequestV40, 0, len(dsrs))
+		for _, dsr := range dsrs {
+			downgraded = append(downgraded, dsr.Downgrade().Downgrade())
+		}
+		api.WriteResp(w, r, downgraded)
 		return
 	}
 
 	downgraded := make([]tc.DeliveryServiceRequestNullable, 0, len(dsrs))
 	for _, dsr := range dsrs {
-		downgraded = append(downgraded, dsr.Downgrade())
+		downgraded = append(downgraded, dsr.Downgrade().Downgrade().Downgrade())
 	}
 
 	api.WriteResp(w, r, downgraded)
@@ -307,15 +312,11 @@ func Get(w http.ResponseWriter, r *http.Request) {
 
 // isTenantAuthorized ensures the user is authorized on the DSR's
 // DeliveryService's Tenant, as appropriate to the change type.
-func isTenantAuthorized(dsr tc.DeliveryServiceRequestV40, inf *api.APIInfo) (bool, error) {
+func isTenantAuthorized(dsr tc.DeliveryServiceRequestV5, inf *api.APIInfo) (bool, error) {
 	if dsr.Requested != nil && (dsr.ChangeType == tc.DSRChangeTypeUpdate || dsr.ChangeType == tc.DSRChangeTypeCreate) {
-		if dsr.Requested.TenantID == nil {
-			log.Debugf("requested.tenantID is nil")
-			return false, errors.New("requested.tenantID is nil")
-		}
-		ok, err := tenant.IsResourceAuthorizedToUserTx(*dsr.Requested.TenantID, inf.User, inf.Tx.Tx)
+		ok, err := tenant.IsResourceAuthorizedToUserTx(dsr.Requested.TenantID, inf.User, inf.Tx.Tx)
 		if err != nil {
-			err = fmt.Errorf("requested: %v", err)
+			err = fmt.Errorf("requested: %w", err)
 		}
 		if !ok || err != nil {
 			return ok, err
@@ -328,19 +329,15 @@ func isTenantAuthorized(dsr tc.DeliveryServiceRequestV40, inf *api.APIInfo) (boo
 		return true, nil
 	}
 
-	if ds.TenantID == nil {
-		log.Debugf("original.tenantID is nil")
-		return false, errors.New("original.tenantID is nil")
-	}
-	ok, err := tenant.IsResourceAuthorizedToUserTx(*ds.TenantID, inf.User, inf.Tx.Tx)
+	ok, err := tenant.IsResourceAuthorizedToUserTx(ds.TenantID, inf.User, inf.Tx.Tx)
 	if err != nil {
-		err = fmt.Errorf("original: %v", err)
+		err = fmt.Errorf("original: %w", err)
 	}
 	return ok, err
 }
 
 // Warning: this assumes inf isn't nil, and neither is dsr, inf.Tx or inf.User or inf.Tx.Tx.
-func insert(dsr *tc.DeliveryServiceRequestV40, inf *api.APIInfo) (int, error, error) {
+func insert(dsr *tc.DeliveryServiceRequestV5, inf *api.APIInfo) (int, error, error) {
 	dsr.Author = inf.User.UserName
 	dsr.LastEditedBy = inf.User.UserName
 	if dsr.ChangeType != tc.DSRChangeTypeDelete {
@@ -369,14 +366,8 @@ func insert(dsr *tc.DeliveryServiceRequestV40, inf *api.APIInfo) (int, error, er
 			sysErr = fmt.Errorf("too many Delivery Services with XMLID '%s'; want: 1, got: %d", dsr.XMLID, len(originals))
 			return http.StatusInternalServerError, nil, sysErr
 		}
-		dsr.Original = new(tc.DeliveryServiceV4)
-		*dsr.Original = originals[0]
-		if inf.Version.Major >= 4 && inf.Version.Minor >= 0 {
-			*dsr.Original = dsr.Original.RemoveLD1AndLD2()
-			if dsr.Requested != nil {
-				*dsr.Requested = dsr.Requested.RemoveLD1AndLD2()
-			}
-		}
+		dsr.Original = new(tc.DeliveryServiceV5)
+		*dsr.Original = originals[0].DS
 	}
 	return http.StatusOK, nil, nil
 }
@@ -420,14 +411,14 @@ func (d dsrManipulationResult) String() string {
 	return builder.String()
 }
 
-func createV4(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result dsrManipulationResult) {
+func createV5(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result dsrManipulationResult) {
 	tx := inf.Tx.Tx
-	var dsr tc.DeliveryServiceRequestV40
+	var dsr tc.DeliveryServiceRequestV5
 	if err := json.NewDecoder(r.Body).Decode(&dsr); err != nil {
-		api.HandleErr(w, r, tx, http.StatusBadRequest, fmt.Errorf("decoding: %v", err), nil)
+		api.HandleErr(w, r, tx, http.StatusBadRequest, fmt.Errorf("decoding: %w", err), nil)
 		return
 	}
-	if userErr, sysErr := validateV4(dsr, tx); userErr != nil || sysErr != nil {
+	if userErr, sysErr := validateV5(dsr, tx); userErr != nil || sysErr != nil {
 		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, sysErr)
 		return
 	}
@@ -459,19 +450,11 @@ func createV4(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result
 		return
 	}
 	if dsr.Original != nil {
-		if dsr.Original.LongDesc1 != nil || dsr.Original.LongDesc2 != nil {
-			api.HandleErr(w, r, tx, http.StatusBadRequest, errors.New("the longDesc1 and longDesc2 fields are no longer supported in API 4.0 onwards"), nil)
-			return
-		}
 		if len(dsr.Original.TLSVersions) < 1 {
 			dsr.Original.TLSVersions = nil
 		}
 	}
 	if dsr.Requested != nil {
-		if dsr.Requested.LongDesc1 != nil || dsr.Requested.LongDesc2 != nil {
-			api.HandleErr(w, r, tx, http.StatusBadRequest, errors.New("the longDesc1 and longDesc2 fields are no longer supported in API 4.0 onwards"), nil)
-			return
-		}
 		if len(dsr.Requested.TLSVersions) < 1 {
 			dsr.Requested.TLSVersions = nil
 		}
@@ -482,7 +465,7 @@ func createV4(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result
 		return
 	}
 
-	w.Header().Set("Location", fmt.Sprintf("/api/%d.%d/deliveryservice_requests/%d", inf.Version.Major, inf.Version.Minor, *dsr.ID))
+	w.Header().Set(rfc.Location, fmt.Sprintf("/api/%s/deliveryservice_requests/%d", inf.Version, *dsr.ID))
 	w.WriteHeader(http.StatusCreated)
 	api.WriteRespAlertObj(w, r, tc.SuccessLevel, "Delivery Service request created", dsr)
 
@@ -494,23 +477,102 @@ func createV4(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result
 	return
 }
 
+func createV4(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result dsrManipulationResult) {
+	tx := inf.Tx.Tx
+	var dsr tc.DeliveryServiceRequestV4
+	if err := json.NewDecoder(r.Body).Decode(&dsr); err != nil {
+		api.HandleErr(w, r, tx, http.StatusBadRequest, fmt.Errorf("decoding: %w", err), nil)
+		return
+	}
+	if userErr, sysErr := validateV4(dsr, tx); userErr != nil || sysErr != nil {
+		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, sysErr)
+		return
+	}
+
+	if dsr.Status != tc.RequestStatusDraft && dsr.Status != tc.RequestStatusSubmitted {
+		userErr := fmt.Errorf("invalid initial request status '%s' - must be '%s' or '%s'", dsr.Status, tc.RequestStatusDraft, tc.RequestStatusSubmitted)
+		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
+		return
+	}
+
+	upgraded := dsr.Upgrade()
+	ok, err := isTenantAuthorized(upgraded, inf)
+	if err != nil {
+		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, err)
+		return
+	}
+	if !ok {
+		api.HandleErr(w, r, tx, http.StatusForbidden, errors.New("not authorized on this tenant"), nil)
+		return
+	}
+
+	upgraded.SetXMLID()
+	if ok, err = dbhelpers.DSRExistsWithXMLID(upgraded.XMLID, tx); err != nil {
+		err = fmt.Errorf("checking for existence of DSR with xmlid '%s'", upgraded.XMLID)
+
+		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, err)
+		return
+	} else if ok {
+		userErr := fmt.Errorf("an open Delivery Service Request for XMLID '%s' already exists", upgraded.XMLID)
+		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
+		return
+	}
+	if upgraded.Original != nil {
+		if len(upgraded.Original.TLSVersions) < 1 {
+			upgraded.Original.TLSVersions = nil
+		}
+	}
+	if upgraded.Requested != nil {
+		if len(upgraded.Requested.TLSVersions) < 1 {
+			upgraded.Requested.TLSVersions = nil
+		}
+	}
+	errCode, userErr, sysErr := insert(&upgraded, inf)
+	if userErr != nil || sysErr != nil {
+		api.HandleErr(w, r, tx, errCode, userErr, sysErr)
+		return
+	}
+
+	dsr = upgraded.Downgrade()
+
+	w.Header().Set(rfc.Location, fmt.Sprintf("/api/%s/deliveryservice_requests/%d", inf.Version, *dsr.ID))
+	w.WriteHeader(http.StatusCreated)
+	if inf.Version.Minor >= 1 {
+		api.WriteRespAlertObj(w, r, tc.SuccessLevel, "Delivery Service request created", dsr)
+	} else {
+		api.WriteRespAlertObj(w, r, tc.SuccessLevel, "Delivery Service request created", dsr.Downgrade())
+	}
+
+	result.Successful = true
+	result.Assignee = dsr.Assignee
+	result.XMLID = dsr.XMLID
+	result.ChangeType = dsr.ChangeType
+	result.Action = api.Created
+	return
+}
+
 func createLegacy(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result dsrManipulationResult) {
 	tx := inf.Tx.Tx
 	var dsr tc.DeliveryServiceRequestNullable
 	if err := json.NewDecoder(r.Body).Decode(&dsr); err != nil {
-		userErr := fmt.Errorf("decoding: %v", err)
+		userErr := fmt.Errorf("decoding: %w", err)
 		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
 		return
 	}
-	if err := validateLegacy(dsr, tx); err != nil {
-		api.HandleErr(w, r, tx, http.StatusBadRequest, err, nil)
+	userErr, sysErr := validateLegacy(dsr, tx)
+	if sysErr != nil {
+		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
+		return
+	}
+	if userErr != nil {
+		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
 		return
 	}
 
-	upgraded := dsr.Upgrade()
+	upgraded := dsr.Upgrade().Upgrade().Upgrade()
 	authorized, err := isTenantAuthorized(upgraded, inf)
 	if err != nil {
-		sysErr := fmt.Errorf("checking tenant authorized: %v", err)
+		sysErr := fmt.Errorf("checking tenant authorized: %w", err)
 		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
 		return
 	}
@@ -542,7 +604,7 @@ func createLegacy(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (res
 	XMLID := *ds.XMLID
 	active, err := isActiveRequest(inf.Tx, XMLID)
 	if err != nil {
-		sysErr := fmt.Errorf("checking request active: %v", err)
+		sysErr := fmt.Errorf("checking request active: %w", err)
 		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
 		return
 	}
@@ -558,14 +620,14 @@ func createLegacy(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (res
 		return
 	}
 
-	api.WriteRespAlertObj(w, r, tc.SuccessLevel, "Delivery Service request created", upgraded.Downgrade())
+	api.WriteRespAlertObj(w, r, tc.SuccessLevel, "Delivery Service request created", upgraded.Downgrade().Downgrade().Downgrade())
 
 	result.Successful = true
 	result.Assignee = dsr.Assignee
 	result.XMLID = upgraded.XMLID
 	result.ChangeType = upgraded.ChangeType
 	result.Action = api.Created
-	return
+	return result
 }
 
 // Post is the handler for POST requests to /deliveryservice_requests.
@@ -590,9 +652,14 @@ func Post(w http.ResponseWriter, r *http.Request) {
 	}
 
 	var result dsrManipulationResult
-	if version.Major >= 4 {
+	switch version.Major {
+	default:
+		fallthrough
+	case 5:
+		result = createV5(w, r, inf)
+	case 4:
 		result = createV4(w, r, inf)
-	} else {
+	case 3:
 		result = createLegacy(w, r, inf)
 	}
 
@@ -603,7 +670,6 @@ func Post(w http.ResponseWriter, r *http.Request) {
 
 // Delete is the handler for DELETE requests to /deliveryservice_requests.
 func Delete(w http.ResponseWriter, r *http.Request) {
-	var omitExtraLongDescFields bool
 	inf, userErr, sysErr, errCode := api.NewInfo(r, []string{"id"}, []string{"id"})
 	tx := inf.Tx.Tx
 	if userErr != nil || sysErr != nil {
@@ -613,8 +679,7 @@ func Delete(w http.ResponseWriter, r *http.Request) {
 	defer inf.Close()
 
 	// Middleware should've already handled this, so idk why this is a pointer at all tbh
-	version := inf.Version
-	if version == nil {
+	if inf.Version == nil {
 		middleware.NotImplementedHandler().ServeHTTP(w, r)
 		return
 	}
@@ -624,12 +689,9 @@ func Delete(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 
-	if version.Major >= 4 && version.Minor >= 0 {
-		omitExtraLongDescFields = true
-	}
-	var dsr tc.DeliveryServiceRequestV40
+	var dsr tc.DeliveryServiceRequestV5
 	if err := inf.Tx.QueryRowx(selectQuery+"WHERE r.id=$1", inf.IntParams["id"]).StructScan(&dsr); err != nil {
-		if err == sql.ErrNoRows {
+		if errors.Is(err, sql.ErrNoRows) {
 			errCode = http.StatusNotFound
 			userErr = fmt.Errorf("no such Delivery Service Request: #%d", inf.IntParams["id"])
 			sysErr = nil
@@ -653,12 +715,12 @@ func Delete(w http.ResponseWriter, r *http.Request) {
 
 	result, err := tx.Exec(deleteQuery, inf.IntParams["id"])
 	if err != nil {
-		sysErr = fmt.Errorf("deleting DSR #%d: %v", inf.IntParams["id"], err)
+		sysErr = fmt.Errorf("deleting DSR #%d: %w", inf.IntParams["id"], err)
 		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
 		return
 	}
 	if affected, err := result.RowsAffected(); err != nil {
-		sysErr = fmt.Errorf("checking affected rows: %v", err)
+		sysErr = fmt.Errorf("checking affected rows: %w", err)
 		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
 		return
 	} else if affected != 1 {
@@ -669,9 +731,9 @@ func Delete(w http.ResponseWriter, r *http.Request) {
 
 	if dsr.IsOpen() && dsr.ChangeType != tc.DSRChangeTypeCreate {
 		if dsr.ChangeType == tc.DSRChangeTypeDelete && dsr.Original != nil && dsr.Original.ID != nil {
-			errCode, userErr, sysErr = getOriginals([]int{*dsr.Original.ID}, inf.Tx, map[int][]*tc.DeliveryServiceRequestV4{*dsr.Original.ID: {&dsr}}, omitExtraLongDescFields)
+			errCode, userErr, sysErr = getOriginals([]int{*dsr.Original.ID}, inf.Tx, map[int][]*tc.DeliveryServiceRequestV5{*dsr.Original.ID: {&dsr}})
 		} else if dsr.ChangeType == tc.DSRChangeTypeUpdate && dsr.Requested != nil && dsr.Requested.ID != nil {
-			errCode, userErr, sysErr = getOriginals([]int{*dsr.Requested.ID}, inf.Tx, map[int][]*tc.DeliveryServiceRequestV4{*dsr.Requested.ID: {&dsr}}, omitExtraLongDescFields)
+			errCode, userErr, sysErr = getOriginals([]int{*dsr.Requested.ID}, inf.Tx, map[int][]*tc.DeliveryServiceRequestV5{*dsr.Requested.ID: {&dsr}})
 		}
 
 		if userErr != nil || sysErr != nil {
@@ -681,10 +743,19 @@ func Delete(w http.ResponseWriter, r *http.Request) {
 	}
 
 	var resp interface{}
-	if inf.Version.Major >= 4 {
+	switch inf.Version.Major {
+	default:
+		fallthrough
+	case 5:
 		resp = dsr
-	} else {
-		resp = dsr.Downgrade()
+	case 4:
+		if inf.Version.Minor >= 1 {
+			resp = dsr.Downgrade()
+		} else {
+			resp = dsr.Downgrade().Downgrade()
+		}
+	case 3:
+		resp = dsr.Downgrade().Downgrade().Downgrade()
 	}
 
 	api.WriteRespAlertObj(w, r, tc.SuccessLevel, fmt.Sprintf("Delivery Service Request #%d deleted", inf.IntParams["id"]), resp)
@@ -699,14 +770,14 @@ func Delete(w http.ResponseWriter, r *http.Request) {
 	inf.CreateChangeLog(res.String())
 }
 
-func putV40(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result dsrManipulationResult) {
+func putV50(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result dsrManipulationResult) {
 	tx := inf.Tx.Tx
-	var dsr tc.DeliveryServiceRequestV40
+	var dsr tc.DeliveryServiceRequestV5
 	if err := json.NewDecoder(r.Body).Decode(&dsr); err != nil {
-		api.HandleErr(w, r, tx, http.StatusBadRequest, fmt.Errorf("decoding: %v", err), nil)
+		api.HandleErr(w, r, tx, http.StatusBadRequest, fmt.Errorf("decoding: %w", err), nil)
 		return
 	}
-	if userErr, sysErr := validateV4(dsr, tx); userErr != nil || sysErr != nil {
+	if userErr, sysErr := validateV5(dsr, tx); userErr != nil || sysErr != nil {
 		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, sysErr)
 		return
 	}
@@ -753,25 +824,14 @@ func putV40(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result ds
 		dsr.Status,
 		inf.IntParams["id"],
 	}
-	if dsr.Original != nil {
-		if dsr.Original.LongDesc1 != nil || dsr.Original.LongDesc2 != nil {
-			api.HandleErr(w, r, tx, http.StatusBadRequest, errors.New("the longDesc1 and longDesc2 fields are no longer supported in API 4.0 onwards"), nil)
-			return
-		}
-	}
-	if dsr.Requested != nil {
-		if dsr.Requested.LongDesc1 != nil || dsr.Requested.LongDesc2 != nil {
-			api.HandleErr(w, r, tx, http.StatusBadRequest, errors.New("the longDesc1 and longDesc2 fields are no longer supported in API 4.0 onwards"), nil)
-			return
-		}
-	}
+
 	if err := tx.QueryRow(updateQuery, args...).Scan(&dsr.CreatedAt, &dsr.LastUpdated); err != nil {
 		var userErr, sysErr error
 		var errCode int
-		if err == sql.ErrNoRows {
+		if errors.Is(err, sql.ErrNoRows) {
 			userErr = fmt.Errorf("no such Delivery Service Request: #%d", inf.IntParams["id"])
 			errCode = http.StatusNotFound
-			sysErr = fmt.Errorf("running update query for Delivery Service Requests: %v", err)
+			sysErr = fmt.Errorf("running update query for Delivery Service Requests: %w", err)
 		} else {
 			userErr, sysErr, errCode = api.ParseDBError(err)
 		}
@@ -780,6 +840,130 @@ func putV40(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result ds
 	}
 	dsr.SetXMLID()
 
+	if dsr.ChangeType == tc.DSRChangeTypeUpdate {
+		query := deliveryservice.SelectDeliveryServicesQuery + `WHERE xml_id=:XMLID`
+		originals, userErr, sysErr, errCode := deliveryservice.GetDeliveryServices(query, map[string]interface{}{"XMLID": dsr.XMLID}, inf.Tx)
+		if userErr != nil || sysErr != nil {
+			api.HandleErr(w, r, tx, errCode, userErr, sysErr)
+			return
+		}
+		if len(originals) < 1 {
+			userErr = fmt.Errorf("cannot update non-existent Delivery Service '%s'", dsr.XMLID)
+			api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
+			return
+		}
+		if len(originals) > 1 {
+			sysErr = fmt.Errorf("too many Delivery Services with XMLID '%s'; want: 1, got: %d", dsr.XMLID, len(originals))
+			api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
+			return
+		}
+		dsr.Original = new(tc.DeliveryServiceV5)
+		*dsr.Original = originals[0].DS
+	}
+
+	api.WriteRespAlertObj(w, r, tc.SuccessLevel, fmt.Sprintf("Delivery Service Request #%d updated", inf.IntParams["id"]), dsr)
+	result.Successful = true
+	result.Action = "Updated"
+	result.Assignee = dsr.Assignee
+	result.ChangeType = dsr.ChangeType
+	result.XMLID = dsr.XMLID
+	return
+}
+
+func putV4(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result dsrManipulationResult) {
+	tx := inf.Tx.Tx
+	var dsr tc.DeliveryServiceRequestV4
+	var dsrV40 tc.DeliveryServiceRequestV40
+
+	if inf.Version.Minor == 0 {
+		if err := json.NewDecoder(r.Body).Decode(&dsrV40); err != nil {
+			api.HandleErr(w, r, tx, http.StatusBadRequest, fmt.Errorf("decoding: %v", err), nil)
+			return
+		}
+		dsr = dsrV40.Upgrade()
+	} else {
+		if err := json.NewDecoder(r.Body).Decode(&dsr); err != nil {
+			api.HandleErr(w, r, tx, http.StatusBadRequest, fmt.Errorf("decoding: %v", err), nil)
+			return
+		}
+		dsrV40 = dsr.Downgrade()
+	}
+	if userErr, sysErr := validateV4(dsr, tx); userErr != nil || sysErr != nil {
+		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, sysErr)
+		return
+	}
+
+	if dsr.Status != tc.RequestStatusDraft && dsr.Status != tc.RequestStatusSubmitted {
+		userErr := fmt.Errorf("cannot change DeliveryServiceRequest status to '%s'", dsr.Status)
+		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
+		return
+	}
+
+	if dsr.ChangeType != tc.DSRChangeTypeDelete {
+		dsr.Original = nil
+	} else {
+		dsr.Requested = nil
+	}
+
+	upgraded := dsr.Upgrade()
+	authorized, err := isTenantAuthorized(upgraded, inf)
+	if err != nil {
+		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, err)
+		return
+	}
+	if !authorized {
+		api.HandleErr(w, r, tx, http.StatusForbidden, errors.New("not authorized on this tenant"), nil)
+		return
+	}
+
+	upgraded.LastEditedBy = inf.User.UserName
+	upgraded.LastEditedByID = new(int)
+	*upgraded.LastEditedByID = inf.User.ID
+
+	if upgraded.Requested != nil && len(upgraded.Requested.TLSVersions) < 1 {
+		upgraded.Requested.TLSVersions = nil
+	}
+	if upgraded.Original != nil && len(upgraded.Original.TLSVersions) < 1 {
+		upgraded.Original.TLSVersions = nil
+	}
+
+	args := []interface{}{
+		upgraded.AssigneeID,
+		upgraded.ChangeType,
+		inf.User.ID,
+		upgraded.Requested,
+		upgraded.Original,
+		upgraded.Status,
+		inf.IntParams["id"],
+	}
+	if dsr.Original != nil {
+		if dsr.Original.LongDesc1 != nil || dsr.Original.LongDesc2 != nil {
+			api.HandleErr(w, r, tx, http.StatusBadRequest, errors.New("the longDesc1 and longDesc2 fields are no longer supported in API 4.0 onwards"), nil)
+			return
+		}
+	}
+	if dsr.Requested != nil {
+		if dsr.Requested.LongDesc1 != nil || dsr.Requested.LongDesc2 != nil {
+			api.HandleErr(w, r, tx, http.StatusBadRequest, errors.New("the longDesc1 and longDesc2 fields are no longer supported in API 4.0 onwards"), nil)
+			return
+		}
+	}
+	if err := tx.QueryRow(updateQuery, args...).Scan(&upgraded.CreatedAt, &upgraded.LastUpdated); err != nil {
+		var userErr, sysErr error
+		var errCode int
+		if errors.Is(err, sql.ErrNoRows) {
+			userErr = fmt.Errorf("no such Delivery Service Request: #%d", inf.IntParams["id"])
+			errCode = http.StatusNotFound
+			sysErr = fmt.Errorf("running update query for Delivery Service Requests: %w", err)
+		} else {
+			userErr, sysErr, errCode = api.ParseDBError(err)
+		}
+		api.HandleErr(w, r, tx, errCode, userErr, sysErr)
+		return
+	}
+	upgraded.SetXMLID()
+	dsr.XMLID = upgraded.XMLID
+
 	if dsr.ChangeType == tc.DSRChangeTypeUpdate {
 		query := deliveryservice.SelectDeliveryServicesQuery + `WHERE xml_id=:XMLID`
 		originals, userErr, sysErr, errCode := deliveryservice.GetDeliveryServices(query, map[string]interface{}{"XMLID": dsr.XMLID}, inf.Tx)
@@ -798,7 +982,7 @@ func putV40(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result ds
 			return
 		}
 		dsr.Original = new(tc.DeliveryServiceV4)
-		*dsr.Original = originals[0]
+		*dsr.Original = originals[0].DS.Downgrade()
 		*dsr.Original = dsr.Original.RemoveLD1AndLD2()
 		if dsr.Requested != nil {
 			*dsr.Requested = dsr.Requested.RemoveLD1AndLD2()
@@ -818,12 +1002,17 @@ func putLegacy(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result
 	tx := inf.Tx.Tx
 	var dsr tc.DeliveryServiceRequestNullable
 	if err := json.NewDecoder(r.Body).Decode(&dsr); err != nil {
-		userErr := fmt.Errorf("decoding: %v", err)
+		userErr := fmt.Errorf("decoding: %w", err)
 		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
 		return
 	}
-	if err := validateLegacy(dsr, tx); err != nil {
-		api.HandleErr(w, r, tx, http.StatusBadRequest, err, nil)
+	userErr, sysErr := validateLegacy(dsr, tx)
+	if sysErr != nil {
+		api.HandleErr(w, r, tx, http.StatusInternalServerError, nil, sysErr)
+		return
+	}
+	if userErr != nil {
+		api.HandleErr(w, r, tx, http.StatusBadRequest, userErr, nil)
 		return
 	}
 
@@ -838,7 +1027,7 @@ func putLegacy(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result
 	dsr.LastEditedByID = new(tc.IDNoMod)
 	*dsr.LastEditedByID = tc.IDNoMod(inf.User.ID)
 
-	upgraded := dsr.Upgrade()
+	upgraded := dsr.Upgrade().Upgrade().Upgrade()
 
 	authorized, err := isTenantAuthorized(upgraded, inf)
 	if err != nil {
@@ -862,7 +1051,7 @@ func putLegacy(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result
 	if err := tx.QueryRow(updateQuery, args...).Scan(&dsr.CreatedAt, &dsr.LastUpdated); err != nil {
 		var errCode int
 		var userErr, sysErr error
-		if err == sql.ErrNoRows {
+		if errors.Is(err, sql.ErrNoRows) {
 			errCode = http.StatusNotFound
 			userErr = fmt.Errorf("no such Delivery Service Request: #%d", inf.IntParams["id"])
 			sysErr = nil
@@ -873,6 +1062,7 @@ func putLegacy(w http.ResponseWriter, r *http.Request, inf *api.APIInfo) (result
 		return
 	}
 	upgraded.SetXMLID()
+	dsr.XMLID = &upgraded.XMLID
 
 	api.WriteRespAlertObj(w, r, tc.SuccessLevel, fmt.Sprintf("Delivery Service Request #%d updated", inf.IntParams["id"]), dsr)
 	result.Action = api.Updated
@@ -894,8 +1084,7 @@ func Put(w http.ResponseWriter, r *http.Request) {
 	defer inf.Close()
 
 	// Middleware should've already handled this, so idk why this is a pointer at all tbh
-	version := inf.Version
-	if version == nil {
+	if inf.Version == nil {
 		middleware.NotImplementedHandler().ServeHTTP(w, r)
 		return
 	}
@@ -913,9 +1102,9 @@ func Put(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 
-	var current tc.DeliveryServiceRequestV40
+	var current tc.DeliveryServiceRequestV5
 	if err := inf.Tx.QueryRowx(selectQuery+"WHERE r.id=$1", id).StructScan(&current); err != nil {
-		if err == sql.ErrNoRows {
+		if errors.Is(err, sql.ErrNoRows) {
 			errCode = http.StatusNotFound
 			userErr = fmt.Errorf("no such Delivery Service Request: #%d", inf.IntParams["id"])
 			sysErr = nil
@@ -943,9 +1132,14 @@ func Put(w http.ResponseWriter, r *http.Request) {
 	}
 
 	var result dsrManipulationResult
-	if inf.Version.Major >= 4 {
-		result = putV40(w, r, inf)
-	} else {
+	switch inf.Version.Major {
+	default:
+		fallthrough
+	case 5:
+		result = putV50(w, r, inf)
+	case 4:
+		result = putV4(w, r, inf)
+	case 3:
 		result = putLegacy(w, r, inf)
 	}
 
