@@ -28,13 +28,13 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/lib/go-util"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/api"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/auth"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/dbhelpers"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/tenant"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/api"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/auth"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/dbhelpers"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/tenant"
 	"github.com/lib/pq"
 )
 
@@ -74,11 +74,11 @@ WHERE d.id IN (
 	FROM deliveryservice_server dss
 	INNER JOIN deliveryservice d ON d.id = dss.deliveryservice
 	WHERE dss.server=$1
-	AND d.active
+	AND d.active = $2
 )
-AND NOT (dss.deliveryservice = ANY($2::BIGINT[]))
-AND (st.name = '` + string(tc.CacheStatusOnline) + `' OR st.name = '` + string(tc.CacheStatusReported) + `')
-AND t.name LIKE $3
+AND NOT (dss.deliveryservice = ANY($3::BIGINT[]))
+AND (st.name = $4 OR st.name = $5)
+AND t.name LIKE $6
 GROUP BY d.id, d.multi_site_origin, d.topology
 HAVING COUNT(dss.server) = 1
 `
@@ -96,7 +96,7 @@ func checkForLastServerInActiveDeliveryServices(serverID int, serverType string,
 		// by definition, only EDGE-type or ORG-type servers can be assigned
 		return violations, nil
 	}
-	rows, err := tx.Query(lastServerInActiveDeliveryServicesQuery, serverID, pq.Array(dsIDs), like)
+	rows, err := tx.Query(lastServerInActiveDeliveryServicesQuery, serverID, tc.DSActiveStateActive, pq.Array(dsIDs), tc.CacheStatusOnline, tc.CacheStatusReported, like)
 	if err != nil {
 		return violations, fmt.Errorf("querying: %v", err)
 	}
@@ -305,12 +305,12 @@ func ValidateDSCapabilities(dsIDs []int, serverName string, tx *sql.Tx) (error,
 	for id, caps := range dsCaps {
 		for _, dsrc := range caps {
 			if !util.ContainsStr(sCaps, dsrc) {
-				return errors.New(fmt.Sprintf("cache %s cannot assign delivery service %d without having the required delivery service capabilities: %v", serverName, id, dsCaps)), nil, http.StatusBadRequest
+				return errors.New(fmt.Sprintf("cache %s cannot assign delivery service %d without having the required delivery service capabilities: %v", serverName, id, caps)), nil, http.StatusBadRequest
 			}
 		}
 	}
 
-	return nil, nil, 0
+	return nil, nil, http.StatusOK
 }
 
 func assignDeliveryServicesToServer(server int, dses []int, replace bool, tx *sql.Tx) ([]int, error) {
