@@ -16,7 +16,7 @@
 package client
 
 import (
-	"github.com/apache/trafficcontrol/traffic_ops/toclientlib"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/toclientlib"
 )
 
 const (
