@@ -17,13 +17,26 @@
  * under the License.
  */
 
-var TableCDNDeliveryServicesController = function(cdn, deliveryServices, filter, $controller, $scope) {
+function TableCDNDeliveryServicesController(cdn, deliveryServices, steeringTargets, filter, $controller, $scope) {
 
 	// extends the TableDeliveryServicesController to inherit common methods
-	angular.extend(this, $controller('TableDeliveryServicesController', { tableName: 'cdnDS', deliveryServices: deliveryServices, filter: filter, $scope: $scope }));
+	angular.extend(this, $controller('TableDeliveryServicesController', { tableName: 'cdnDS', deliveryServices: deliveryServices, steeringTargets: steeringTargets, filter: filter, $scope: $scope }));
 
 	$scope.cdn = cdn;
+	$scope.breadCrumbs = [
+		{
+			href: "#!/cdns",
+			text: "CDNs"
+		},
+		{
+			href: `#!/cdns/${cdn.id}`,
+			text: cdn.name
+		},
+		{
+			text: "Delivery Services"
+		}
+	];
 };
 
-TableCDNDeliveryServicesController.$inject = ['cdn', 'deliveryServices', 'filter', '$controller', '$scope'];
+TableCDNDeliveryServicesController.$inject = ['cdn', 'deliveryServices', 'steeringTargets', 'filter', '$controller', '$scope'];
 module.exports = TableCDNDeliveryServicesController;
