@@ -17,13 +17,26 @@
  * under the License.
  */
 
-var TableTenantDeliveryServicesController = function(tenant, deliveryServices, filter, $controller, $scope) {
+var TableTenantDeliveryServicesController = function(tenant, deliveryServices, steeringTargets, filter, $controller, $scope) {
 
 	// extends the TableDeliveryServicesController to inherit common methods
-	angular.extend(this, $controller('TableDeliveryServicesController', { tableName: 'tenantDS', deliveryServices: deliveryServices, filter: filter, $scope: $scope }));
+	angular.extend(this, $controller('TableDeliveryServicesController', { tableName: 'tenantDS', deliveryServices: deliveryServices, steeringTargets: steeringTargets, filter: filter, $scope: $scope }));
 
 	$scope.tenant = tenant;
+	$scope.breadCrumbs = [
+		{
+			href: "#!/tenants",
+			text: "Tenants"
+		},
+		{
+			getText: () => tenant.name,
+			getHref: () => `#!/tenants/${tenant.id}`
+		},
+		{
+			text: "Delivery Services"
+		}
+	];
 };
 
-TableTenantDeliveryServicesController.$inject = ['tenant', 'deliveryServices', 'filter', '$controller', '$scope'];
+TableTenantDeliveryServicesController.$inject = ['tenant', 'deliveryServices', 'steeringTargets', 'filter', '$controller', '$scope'];
 module.exports = TableTenantDeliveryServicesController;
